#######################################################-
# Project: New York proactive policing and crime
# Last modification: January 7, 2022
# Code author: David Mitre-Becerril
# Paper authors: Aaron Chalfin, David Mitre-Becerril, Morgan C. Williams, Jr.
# Objective: replicate the main findings from Sullivan & O'Keeffe (2017) and conduct
# several robustness checks and testable implications.
# Folder structure: 
# |-- Code
# |-- Data
# |-- Tables-Figures
# Data sources:
# 1) Census_Tracts_2010 (folder)
# Description: shapefile of the 2010 census tracts from New York City, NY.
# Institution/author: GIS Lab, Newman Library, Baruch CUNY.
# Url: https://geodata.lib.berkeley.edu/catalog/nyu-2451-34505  
# 2) nyc_2010pop_2020precincts.csv (file)
# Description: csv file with 2010 census tract population estimates from New York City, NY.
# Institution/author: John Keefe
# Url: https://johnkeefe.net/nyc-police-precinct-and-census-data
# 3) NYPD_Arrests_Data__Historic_.csv (file)
# Description: csv file of every arrest in New York City, NY from 2006 onwards.
# Institution/author: New York City Police Department
# Url: https://data.cityofnewyork.us/Public-Safety/NYPD-Arrests-Data-Historic-/8h9b-rp9u
# 4) NYPD_Complaint_Data_Historic.csv (file)
# Description: csv file of all valid felony, misdemeanor, and violation crimes 
# reported to the New York City Police Department from 2006 onwards.
# Institution/author: New York City Police Department
# Url: https://data.cityofnewyork.us/Public-Safety/NYPD-Complaint-Data-Historic/qgea-i56i
# 5) NYPD_Criminal_Court_Summons__Historic_.csv (file)
# Description: csv file of every criminal summons in New York City from 2006 onwards.
# Institution/author: New York City Police Department
# Url: https://data.cityofnewyork.us/Public-Safety/NYPD-Criminal-Court-Summons-Historic-/sv2w-rv3k
# 6) NYPD_SQF_Data (folder)
# Description: csv/excel files from the NYPD stop, question, and frisk database from 2006 onwards.
# Institution/author: New York City Police Department
# Url: https://data.cityofnewyork.us/Public-Safety/The-Stop-Question-and-Frisk-Data/ftxv-d5ix
#######################################################-

# Clean the global environment
while (!is.null(dev.list())) {
  dev.off(dev.list()["RStudioGD"])
}
# Load all the required packages
ipak <- function(pkg) {
  new.pkg <- pkg[!(pkg %in% installed.packages()[, "Package"])]
  if (length(new.pkg))
    install.packages(new.pkg, dependencies = TRUE, repos = "http://cran.us.r-project.org")
  sapply(pkg, require, character.only = TRUE)
}
ipak(c("rstudioapi",
       "lmtest",
       "sandwich",
       "MASS",
       "data.table",
       "lubridate",
       "cowplot",
       "rnoaa",
       "fixest",
       "readxl",
       "stargazer",
       "sf", 
       "tidyverse"))
options(scipen=999) #to avoid scientific notation when exporting latex tables
# Setup the working directory (to ensure all files are exported/imported correctly)
path <- dirname(rstudioapi::getActiveDocumentContext()$path)
setwd(path)
rm(path, ipak)



###################################################################################-
### Data processing: download and process day-year-city level weather data ########
###################################################################################-
#Download weather data
#https://www.ncei.noaa.gov/data/global-hourly/access/2015/
#https://www.ncei.noaa.gov/data/global-hourly/doc/isd-format-document.pdf
#72503014732 #NEW YORK LAGUARDIA AP 
#72505394728 #NEW YORK CNTRL PK TWR
#74486094789 #NEW YORK JFK INTL AP
weatherlist = list()
counter <- 1
for(X in c(72503014732, 72505394728, 74486094789)){
  for(Z in 2006:2019){
    weatherlist[[counter]] <- lcd(X, year = Z)
    counter <- counter + 1  
  }
}
dfweather <- plyr::rbind.fill(weatherlist)
rm(X, Z, counter, weatherlist)

#Data processing at the day-year-city level
dfweather <- dfweather %>%
  mutate(date = as.Date(date),
         year = year(date),
         month = month(date),
         day = day(date)) %>%
  rename(temperature = hourlydewpointtemperature,
         precipitation = hourlyprecipitation,
         humidity = hourlyrelativehumidity,
         visibility = hourlyvisibility, 
         snow = dailysnowfall) %>%
  select(year, month, day, temperature, precipitation, humidity, visibility, snow) %>%
  group_by(year, month, day) %>%
  mutate_at(vars(temperature:snow), as.numeric) %>%
  summarise(temperature = mean(temperature, na.rm = TRUE),
            precipitation = sum(precipitation, na.rm = TRUE),
            humidity = mean(humidity, na.rm = TRUE),
            visibility = mean(visibility, na.rm = TRUE), 
            snow = sum(snow, na.rm = TRUE))
fwrite(dfweather, "../Data/Weather day-year-city data 2006-2019bis.csv")
#dfweather <- fread("../Data/Weather day-year-city data 2006-2019.csv")
rm(dfweather); gc()





################################################################################-
### Data processing: aggregate data to the day-year-precinct level #############
################################################################################-
##### Arrest data ######-
#https://data.cityofnewyork.us/Public-Safety/NYPD-Arrests-Data-Historic-/8h9b-rp9u
#There is one incident in the 27th precinct in the arrest dataset but such precinct does not exist (must be a typo)
#2008, 2012, and 2016 have 366 days.
#for(X in 2006:2019) print(c(X, yday(mdy(paste0("12-31-", X)))))
dfarrest <- fread("../Data/NYPD_Arrests_Data__Historic_.csv",
                  select = c("ARREST_DATE", "KY_CD", "OFNS_DESC", "ARREST_PRECINCT")) %>%
  rename_all(tolower)
df <- expand.grid(year = 2006:2019, month = 1:12, day = 1:31, precinct = unique(dfarrest$arrest_precinct)) %>% 
  filter(!is.na(mdy(paste(month, day, year, sep = "-")))) %>%
  arrange(year, month, day, precinct) %>%
  filter(precinct != 27)
dfarrest <- dfarrest %>%
  mutate(year = year(mdy(arrest_date)), 
         month = month(mdy(arrest_date)),
         day = day(mdy(arrest_date))) %>%
  rename(precinct = arrest_precinct) %>%
  filter(year %in% 2006:2019) %>%
  mutate(crime = "arrest_nonmajor",
         crime = ifelse(ky_cd %in% 101:103, "arrest_murder", crime),
         crime = ifelse(ky_cd %in% 104, "arrest_rape", crime),
         crime = ifelse(ky_cd %in% 105, "arrest_robbery", crime),
         crime = ifelse(ky_cd %in% 106, "arrest_assault", crime),
         crime = ifelse(ky_cd %in% 107, "arrest_burglary", crime),
         crime = ifelse(ky_cd %in% 109, "arrest_theft", crime),
         crime = ifelse(ky_cd %in% 110, "arrest_mvtheft", crime)) %>%
  group_by(year, month, day, precinct, crime) %>%
  summarise(crimes = n()) %>%
  pivot_wider(names_from = crime, values_from = crimes, values_fill = 0) 
df <- df %>% 
  left_join(dfarrest, by = c("year", "month", "day", "precinct")) %>%
  replace(is.na(.), 0)
df <- df %>%
  mutate(arrest_violent = arrest_murder + arrest_rape + arrest_robbery + arrest_assault,
         arrest_property = arrest_theft + arrest_burglary + arrest_mvtheft,
         arrest_major = arrest_violent + arrest_property)
rm(dfarrest); gc()


##### Detailed arrest data: PSB and narcotics arrests ######-
#jurisdiction code == 0 is Patrol
dfarrest <- fread("../Data/NYPD_Arrests_Data__Historic_.csv",
                  select = c("ARREST_DATE", "KY_CD", "OFNS_DESC", "ARREST_PRECINCT", "JURISDICTION_CODE")) %>%
  rename_all(tolower)
dfarrest <- dfarrest %>%
  mutate(year = year(mdy(arrest_date)), 
         month = month(mdy(arrest_date)),
         day = day(mdy(arrest_date))) %>%
  rename(precinct = arrest_precinct) %>%
  filter(year %in% 2006:2019) %>%
  mutate(arrest_patrol = ifelse(jurisdiction_code %in% 0, 1, 0), 
         arrest_drugs = ifelse(ofns_desc %in% "DANGEROUS DRUGS", 1, 0)) %>%
  group_by(year, month, day, precinct) %>%
  summarise_at(vars(arrest_patrol, arrest_drugs), sum, na.rm = TRUE)
df <- df %>% 
  left_join(dfarrest, by = c("year", "month", "day", "precinct")) %>%
  replace(is.na(.), 0)
rm(dfarrest); gc()


##### Crime data ######-
#https://data.cityofnewyork.us/Public-Safety/NYPD-Complaint-Data-Historic/qgea-i56i
dfcrime <- fread("../Data/NYPD_Complaint_Data_Historic.csv", 
                 select = c("CMPLNT_FR_DT", "KY_CD", "ADDR_PCT_CD", "LOC_OF_OCCUR_DESC", 
                            "PREM_TYP_DESC", "OFNS_DESC")) %>%
  rename_all(tolower); gc()
#dfcrime %>% distinct(ky_cd, ofns_desc, prem_typ_desc, loc_of_occur_desc) %>% View()
dfcrime <- dfcrime %>%
  mutate(year = year(mdy(cmplnt_fr_dt)), 
         month = month(mdy(cmplnt_fr_dt)),
         day = day(mdy(cmplnt_fr_dt))) %>%
  rename(precinct = addr_pct_cd) %>%
  filter(year %in% 2006:2019) %>%
  mutate(crime = "crime_nonmajor",
         crime = ifelse(ky_cd %in% 101:103, "crime_murder", crime),
         crime = ifelse(ky_cd %in% 104, "crime_rape", crime),
         crime = ifelse(ky_cd %in% 105, "crime_robbery", crime),
         crime = ifelse(ky_cd %in% 106, "crime_assault", crime),
         crime = ifelse(ky_cd %in% 107, "crime_burglary", crime),
         crime = ifelse(ky_cd %in% 109, "crime_theft", crime),
         crime = ifelse(ky_cd %in% 110, "crime_mvtheft", crime))
#dfcrime %>% filter(loc_of_occur_desc == "INSIDE") %>% distinct(prem_typ_desc, loc_of_occur_desc) %>% View()
#Major crime categories
dfcrime1 <- dfcrime %>%
  group_by(year, month, day, precinct, crime) %>%
  summarise(crimes = n()) %>%
  pivot_wider(names_from = crime, values_from = crimes, values_fill = 0)
#Major crime based on inside/outside a residence location
dfcrime2 <- dfcrime %>%
  mutate(inside = ifelse(loc_of_occur_desc == "INSIDE" & grepl("RESIDENCE.*", prem_typ_desc), 1, 0),
         crime_in = "crime_major_out",
         crime_in = ifelse(crime == "crime_nonmajor" & inside == 1, "crime_nonmajor_in", crime_in),
         crime_in = ifelse(crime == "crime_nonmajor" & inside == 0, "crime_nonmajor_out", crime_in),
         crime_in = ifelse(crime != "crime_nonmajor" & inside == 1, "crime_major_in", crime_in)) %>%
  select(year, month, day, precinct, crime_in) %>%
  group_by(year, month, day, precinct, crime_in) %>%
  summarise(crimes = n()) %>%
  pivot_wider(names_from = crime_in, values_from = crimes, values_fill = 0)
df <- df %>% 
  left_join(dfcrime1, by = c("year", "month", "day", "precinct")) %>%
  left_join(dfcrime2, by = c("year", "month", "day", "precinct")) %>%
  replace(is.na(.), 0)
df <- df %>%
  mutate(crime_violent = crime_murder + crime_rape + crime_robbery + crime_assault,
         crime_property = crime_theft + crime_burglary + crime_mvtheft,
         crime_major = crime_violent + crime_property)
rm(dfcrime, dfcrime1, dfcrime2); gc()


##### Criminal summons data ######-
#https://data.cityofnewyork.us/Public-Safety/NYPD-Criminal-Court-Summons-Historic-/sv2w-rv3k
dfsummons <- fread("../Data/NYPD_Criminal_Court_Summons__Historic_.csv", 
                   select = c("SUMMONS_DATE", "PRECINCT_OF_OCCUR")) %>%
  rename_all(tolower); gc()
dfsummons <- dfsummons %>%
  mutate(year = year(mdy(summons_date)), 
         month = month(mdy(summons_date)),
         day = day(mdy(summons_date))) %>%
  rename(precinct = precinct_of_occur) %>%
  filter(year %in% 2006:2019) %>%
  group_by(year, month, day, precinct) %>%
  summarise(summons = n())
df <- df %>% 
  left_join(dfsummons, by = c("year", "month", "day", "precinct")) %>%
  replace(is.na(.), 0)
rm(dfsummons); gc()


##### Stop, Question, and Frisk data ######-
#https://data.cityofnewyork.us/Public-Safety/The-Stop-Question-and-Frisk-Data/ftxv-d5ix
vfiles <- list.files("../Data/NYPD_SQF_Data", full.names = TRUE) %>%
  grep("20[0-1][0-9]", ., value = TRUE)
dfsqf <- data.frame(year = numeric(), month = numeric(), day = numeric(), 
                    precinct = numeric(), stopfrisk = numeric())
for(X in 1:length(vfiles)){
  if(X %in% c(2:11)){
    dftemp <- fread(vfiles[X]) %>%
      rename_all(tolower) %>%
      mutate(year = year(mdy(datestop)), 
             month = month(mdy(datestop)),
             day = day(mdy(datestop))) %>%
      rename(precinct = pct) %>%
      group_by(year, month, day, precinct) %>%
      summarise(stopfrisk = n())
    dfsqf <- bind_rows(dfsqf, dftemp)
  } else if(X == 1){
    dftemp <- fread(vfiles[X]) %>%
      rename_all(tolower) %>%
      mutate(year = year(ymd(datestop)), 
             month = month(ymd(datestop)),
             day = day(ymd(datestop))) %>%
      rename(precinct = pct) %>%
      group_by(year, month, day, precinct) %>%
      summarise(stopfrisk = n())
    dfsqf <- bind_rows(dfsqf, dftemp)
  } else{
    dftemp <- read_xlsx(vfiles[X]) %>%
      rename_all(tolower) %>%
      mutate(year = year(ymd(stop_frisk_date)), 
             month = month(ymd(stop_frisk_date)),
             day = day(ymd(stop_frisk_date))) %>%
      rename(precinct = stop_location_precinct) %>%
      group_by(year, month, day, precinct) %>%
      summarise(stopfrisk = n())
    dfsqf <- bind_rows(dfsqf, dftemp)
  }
}
df <- df %>% 
  left_join(dfsqf, by = c("year", "month", "day", "precinct")) %>%
  replace(is.na(.), 0)
rm(dfsqf, dftemp, vfiles, X); gc()


##### Merge with weather data ######-
dfweather <- fread("../Data/Weather day-year-city data 2006-2019.csv")
df <- df %>% left_join(dfweather, by = c("year", "month", "day"))
rm(dfweather); gc()


##### Merge with population data ######-
#https://johnkeefe.net/nyc-police-precinct-and-census-data
dfpop <- fread("../Data/nyc_2010pop_2020precincts.csv") %>%
  rename(precinct = precinct_2020,
         population = P0010001) %>%
  select(precinct, population)
df <- df %>% left_join(dfpop, by = c("precinct"))
rm(dfpop); gc()
#Export data
fwrite(df, "../Data/Crime day-year-precinct data 2006-2019.csv")
#df <- fread("../Data/Crime day-year-precinct data 2006-2019.csv")
rm(df); gc()




################################################################################-
### Data processing: aggregate data to the day-year-census tract level #########
################################################################################-
#Import shapefile (original shapefile to use a preferred CRS)
#https://geodata.lib.berkeley.edu/catalog/nyu-2451-34505
sf <- read_sf("../Data/Census_Tracts_2010/Census_Tracts_2010.shp")

##### Arrest data ######-
#https://data.cityofnewyork.us/Public-Safety/NYPD-Arrests-Data-Historic-/8h9b-rp9u
#There is a 27th precinct in the arrest dataset but not in the NYPD precinct website
#2008, 2012, and 2016 have 366 days.
#for(X in 2006:2019) print(c(X, yday(mdy(paste0("12-31-", X)))))
dfarrest <- fread("../Data/NYPD_Arrests_Data__Historic_.csv", 
                  select = c("ARREST_DATE", "KY_CD", "Latitude", "Longitude")) %>%
  rename_all(tolower) %>%
  mutate(year = year(mdy(arrest_date)), 
         month = month(mdy(arrest_date)),
         day = day(mdy(arrest_date))) %>%
  filter(year %in% 2006:2019) %>%
  mutate(crime = "arrest_nonmajor",
         crime = ifelse(ky_cd %in% 101, "arrest_murder", crime),
         crime = ifelse(ky_cd %in% 104, "arrest_rape", crime),
         crime = ifelse(ky_cd %in% 105, "arrest_robbery", crime),
         crime = ifelse(ky_cd %in% 106, "arrest_assault", crime),
         crime = ifelse(ky_cd %in% 107, "arrest_burglary", crime),
         crime = ifelse(ky_cd %in% 109, "arrest_theft", crime),
         crime = ifelse(ky_cd %in% 110, "arrest_mvtheft", crime)) %>%
  select(year, month, day, crime, latitude, longitude) %>%
  filter(complete.cases(latitude, longitude))
#Merge datapoints to tracts shapefile and transform it to a data frame (by year to speed up memory computations)
dftemp1 <- data.frame()
for (X in 2006:2019){
  dftemp2 <- st_as_sf(dfarrest %>% filter(year == X), coords = c("longitude","latitude"), crs = 4326) %>%
    st_join(sf %>% select(tractid)) %>%
    st_set_geometry(NULL) %>% 
    as.data.frame() %>%
    filter(complete.cases(tractid)) %>%
    group_by(year, month, day, tractid, crime) %>%
    summarise(crimes = n()) %>%
    pivot_wider(names_from = crime, values_from = crimes, values_fill = 0) %>%
    mutate(arrest_violent = arrest_murder + arrest_rape + arrest_robbery + arrest_assault,
           arrest_property = arrest_theft + arrest_burglary + arrest_mvtheft,
           arrest_major = arrest_violent + arrest_property) %>%
    as.data.frame()
  dftemp1 <- bind_rows(dftemp1, dftemp2)
  print(paste0("Ready loop: ", X))
}
dfarrest <- dftemp1
rm(dftemp1, dftemp2, X); gc()


##### Detailed arrest data: PSB and narcotics arrests ######-
#jurisdiction code == 0 is Patrol
dfarrest2 <- fread("../Data/NYPD_Arrests_Data__Historic_.csv",
                  select = c("ARREST_DATE", "OFNS_DESC", "JURISDICTION_CODE", "Latitude", "Longitude")) %>%
  rename_all(tolower)
dfarrest2 <- dfarrest2 %>%
  mutate(year = year(mdy(arrest_date)), 
         month = month(mdy(arrest_date)),
         day = day(mdy(arrest_date))) %>%
  filter(year %in% 2006:2019) %>%
  mutate(arrest_patrol = ifelse(jurisdiction_code %in% 0, 1, 0), 
         arrest_drugs = ifelse(ofns_desc %in% "DANGEROUS DRUGS", 1, 0)) %>%
  select(year, month, day, arrest_patrol, arrest_drugs, latitude, longitude) %>%
  filter(complete.cases(latitude, longitude))
#Merge datapoints to tracts shapefile and transform it to a data frame (by year to speed up memory computations)
dftemp1 <- data.frame()
for (X in 2006:2019){
  dftemp2 <- st_as_sf(dfarrest2 %>% filter(year == X), coords = c("longitude","latitude"), crs = 4326) %>%
    st_join(sf %>% select(tractid)) %>%
    st_set_geometry(NULL) %>% 
    as.data.frame() %>%
    filter(complete.cases(tractid)) %>%
    group_by(year, month, day, tractid) %>%
    summarise_at(vars(arrest_patrol, arrest_drugs), sum, na.rm = TRUE) %>%
    as.data.frame()
  dftemp1 <- bind_rows(dftemp1, dftemp2)
  print(paste0("Ready loop: ", X))
}
dfarrest2 <- dftemp1
rm(dftemp1, dftemp2, X); gc()


##### Crime data ######-
#https://data.cityofnewyork.us/Public-Safety/NYPD-Complaint-Data-Historic/qgea-i56i
dfcrime <- fread("../Data/NYPD_Complaint_Data_Historic.csv",
                 select = c("CMPLNT_FR_DT", "KY_CD", "LOC_OF_OCCUR_DESC", "PREM_TYP_DESC",
                            "Latitude", "Longitude")) %>%
  rename_all(tolower) %>%
  mutate(year = year(mdy(cmplnt_fr_dt)), 
         month = month(mdy(cmplnt_fr_dt)),
         day = day(mdy(cmplnt_fr_dt))) %>%
  filter(year %in% 2006:2019) %>%
  mutate(crime = "crime_nonmajor",
         crime = ifelse(ky_cd %in% 101, "crime_murder", crime),
         crime = ifelse(ky_cd %in% 104, "crime_rape", crime),
         crime = ifelse(ky_cd %in% 105, "crime_robbery", crime),
         crime = ifelse(ky_cd %in% 106, "crime_assault", crime),
         crime = ifelse(ky_cd %in% 107, "crime_burglary", crime),
         crime = ifelse(ky_cd %in% 109, "crime_theft", crime),
         crime = ifelse(ky_cd %in% 110, "crime_mvtheft", crime)) %>%
  filter(complete.cases(latitude, longitude))
#Merge datapoints to tracts shapefile and transform it to a data frame (by year to speed up memory computations)
dftemp1 <- data.frame()
for(X in 2006:2019){
  dftemp2 <- st_as_sf(dfcrime %>% filter(year == X), coords = c("longitude","latitude"), crs = 4326) %>%
    st_join(sf %>% select(tractid)) %>%
    st_set_geometry(NULL) %>% 
    as.data.frame() %>%
    filter(complete.cases(tractid))
  dftemp1 <- bind_rows(dftemp1, dftemp2)
  print(paste0("Ready loop: ", X))
}
dfcrime <- dftemp1
rm(dftemp1, dftemp2, X); gc()
#Major crime categories
dfcrime1 <- dfcrime %>%
  group_by(year, month, day, tractid, crime) %>%
  summarise(crimes = n()) %>%
  pivot_wider(names_from = crime, values_from = crimes, values_fill = 0)
#Major crime based on inside/outside a residence location
dfcrime2 <- dfcrime %>%
  mutate(inside = ifelse(loc_of_occur_desc == "INSIDE" & grepl("RESIDENCE.*", prem_typ_desc), 1, 0),
         crime_in = "crime_major_out",
         crime_in = ifelse(crime == "crime_nonmajor" & inside == 1, "crime_nonmajor_in", crime_in),
         crime_in = ifelse(crime == "crime_nonmajor" & inside == 0, "crime_nonmajor_out", crime_in),
         crime_in = ifelse(crime != "crime_nonmajor" & inside == 1, "crime_major_in", crime_in)) %>%
  select(year, month, day, tractid, crime_in) %>%
  group_by(year, month, day, tractid, crime_in) %>%
  summarise(crimes = n()) %>%
  pivot_wider(names_from = crime_in, values_from = crimes, values_fill = 0)
dfcrime <- dfcrime1 %>% 
  left_join(dfcrime2, by = c("year", "month", "day", "tractid")) %>%
  replace(is.na(.), 0)
dfcrime <- dfcrime %>%
  mutate(crime_violent = crime_murder + crime_rape + crime_robbery + crime_assault,
         crime_property = crime_theft + crime_burglary + crime_mvtheft,
         crime_major = crime_violent + crime_property)
rm(dfcrime1, dfcrime2); gc()


##### Criminal summons data ######-
#https://data.cityofnewyork.us/Public-Safety/NYPD-Criminal-Court-Summons-Historic-/sv2w-rv3k
dfsummons <- fread("../Data/NYPD_Criminal_Court_Summons__Historic_.csv", 
                   select = c("SUMMONS_DATE", "Latitude", "Longitude")) %>%
  rename_all(tolower); gc()
dfsummons <- dfsummons %>%
  mutate(year = year(mdy(summons_date)), 
         month = month(mdy(summons_date)),
         day = day(mdy(summons_date))) %>%
  filter(year %in% 2006:2019) %>%
  filter(complete.cases(latitude, longitude))
#Merge datapoints to tracts shapefile and transform it to a data frame (by year to speed up memory computations)
dftemp1 <- data.frame()
for (X in 2006:2019){
  dftemp2 <- st_as_sf(dfsummons %>% filter(year == X), coords = c("longitude","latitude"), crs = 4326) %>%
    st_join(sf %>% select(tractid)) %>%
    st_set_geometry(NULL) %>% 
    as.data.frame() %>%
    filter(complete.cases(tractid)) %>%
    group_by(year, month, day, tractid) %>%
    summarise(summons = n()) %>%
    as.data.frame()
  dftemp1 <- bind_rows(dftemp1, dftemp2)
  print(paste0("Ready loop: ", X))
}
dfsummons <- dftemp1
rm(dftemp1, dftemp2, X); gc()


##### Stop, Question, and Frisk data ######-
#https://data.cityofnewyork.us/Public-Safety/The-Stop-Question-and-Frisk-Data/ftxv-d5ix
vfiles <- list.files("../Data/NYPD_SQF_Data", full.names = TRUE) %>%
  grep("20[0-1][0-9]", ., value = TRUE)
dfsqf <- data.frame(year = numeric(), month = numeric(), day = numeric(), stopfrisk = numeric())
for(X in 1:length(vfiles)){
  if(X %in% c(2:11)){
    dftemp <- fread(vfiles[X]) %>%
      rename_all(tolower) %>%
      mutate(year = year(mdy(datestop)), 
             month = month(mdy(datestop)),
             day = day(mdy(datestop))) %>%
      rename(longitude = xcoord, latitude = ycoord) %>%
      select(year, month, day, longitude, latitude)
  } else if(X == 1){
    dftemp <- fread(vfiles[X]) %>%
      rename_all(tolower) %>%
      mutate(year = year(ymd(datestop)), 
             month = month(ymd(datestop)),
             day = day(ymd(datestop))) %>%
      rename(longitude = xcoord, latitude = ycoord) %>%
      select(year, month, day, longitude, latitude)
  } else{
    dftemp <- read_xlsx(vfiles[X]) %>%
      rename_all(tolower) %>%
      mutate(year = year(ymd(stop_frisk_date)), 
             month = month(ymd(stop_frisk_date)),
             day = day(ymd(stop_frisk_date))) %>%
      rename(longitude = stop_location_x, latitude = stop_location_y) %>%
      select(year, month, day, longitude, latitude)
  }
  dfsqf <- bind_rows(dfsqf, dftemp)
}
rm(dftemp, vfiles)
#Merge datapoints to tracts shapefile and transform it to a data frame (by year to speed up memory computations)
dfsqf <- dfsqf %>%
  filter(complete.cases(latitude, longitude))
dftemp1 <- data.frame()
for (X in 2006:2019){
  dftemp2 <- st_as_sf(dfsqf %>% filter(year == X), coords = c("longitude","latitude"), crs = 2263) %>%
    st_transform(crs = 4326) %>%
    st_join(sf %>% select(tractid)) %>%
    st_set_geometry(NULL) %>% 
    as.data.frame() %>%
    filter(complete.cases(tractid)) %>%
    group_by(year, month, day, tractid) %>%
    summarise(stopfrisk = n()) %>%
    as.data.frame()
  dftemp1 <- bind_rows(dftemp1, dftemp2)
  print(paste0("Ready loop: ", X))
}
dfsqf <- dftemp1
rm(dftemp1, dftemp2, X); gc()

#Create a master dataset for the city
dfweather <- fread("../Data/Weather day-year-city data 2006-2019.csv")
df <- expand.grid(year = 2006:2019, month = 1:12, day = 1:31, tractid = unique(sf$tractid)) %>% 
  filter(!is.na(mdy(paste(month, day, year, sep = "-")))) %>%
  arrange(year, month, day, tractid) %>%
  left_join(dfcrime, by = c("year", "month", "day", "tractid")) %>%
  left_join(dfarrest, by = c("year", "month", "day", "tractid")) %>%
  left_join(dfarrest2, by = c("year", "month", "day", "tractid")) %>%
  left_join(dfsummons, by = c("year", "month", "day", "tractid")) %>%
  left_join(dfsqf, by = c("year", "month", "day", "tractid")) %>%
  left_join(dfweather, by = c("year", "month", "day")) %>%
  replace(is.na(.), 0)
fwrite(df, "../Data/Crime day-year-tract data 2006-2019.csv")
#df <- fread("../Data/Crime day-year-tract data 2006-2019.csv", integer64 = "character")
rm(df, dfcrime, dfarrest, dfarrest2, dfsummons, dfsqf, dfweather, sf); gc()






################################################################################-
### Regression: Negative binomial estimates between t and t-1 ##################
################################################################################-
##### Using data from 01-19-year to 01-18-year+2 (replicate authors sample) ####-
#Import data (all models exclude Central Park precinct)
df <- fread("../Data/Crime day-year-precinct data 2006-2019.csv") %>%
  mutate(date = mdy(paste(month, day, year, sep = "-"))) %>%
  filter(precinct != 22) 
#create indicator variables for series windows
for(X in 2006:2019){
  date1 <- mdy(paste0("01-19-", X)) 
  date2 <- mdy(paste0("01-18-", X+1)) 
  df[,paste0("s", X)] <- ifelse(df$date >= date1 & df$date <= date2, 1, 0)
} 
rm(date1, date2, X)
#create indicator variables for treatment window
for(X in 2006:2019){
  date1 <- mdy(paste0("12-01-", X)) 
  date2 <- mdy(paste0("01-18-", X+1)) 
  df[,paste0("t", X)] <- ifelse(df$date >= date1 & df$date <= date2, 1, 0)
} 
rm(date1, date2, X)
#Estimate the negative binomial regression by year-pairs (t vs t-1)
vnames <- df %>% 
  select(arrest_patrol, arrest_drugs, contains("major"), summons) %>% 
  names()
dfest <- data.frame()
#First loop goes over the outcome variable, the second one over the years-pairs
for(W in vnames){
  for(X in 2006:2017){
    #Create indicator variables
    s1 <- paste0("s", X); s2 <- paste0("s", X+1) 
    t1 <- paste0("t", X); t2 <- paste0("t", X+1) 
    #Subset data
    dftemp <- df %>%
      filter(!!sym(s1) == 1 | !!sym(s2) == 1) %>%
      mutate(series = ifelse(!!sym(s2) == 1, 1, 0), 
             treat = ifelse(!!sym(t1) == 1, 1, 0), 
             treat = ifelse(!!sym(t2) == 1, 1, treat)) 
    #Estimate regression (with and without precinct-fixed effects)
    for(Y in c("precinct.fe", "no.fe")){
        if(Y == "precinct.fe"){
          f <- as.formula(paste0(W, " ~ series*treat + snow + precipitation + temperature | precinct"))
        } else if(Y == "no.fe"){
          f <- as.formula(paste0(W, " ~ series*treat + snow + precipitation + temperature"))
        } 
        nb <- fenegbin(f, data = dftemp, cluster = ~ precinct)
        dfest <- dfest %>% 
          bind_rows(data.frame("outcome" = W, 
                               "period.treat" = paste0("dec", substr(X+1,3,4), "-jan", substr(X+2,3,4)),
                               "fe" = Y,
                               "irr" = exp(nb$coeftable["series:treat","Estimate"]), 
                               "se" = (exp(nb$coefficients)*sqrt(diag(vcov(nb))))["series:treat"], 
                               "ci.low" = exp(nb$coefficients - 1.96*sqrt(diag(vcov(nb))))["series:treat"],
                               "ci.up" = exp(nb$coefficients + 1.96*sqrt(diag(vcov(nb))))["series:treat"]))
      }
    }
  print(paste0("Ready: ", W))
}
rm(W, X, Y, vnames, f, s1, s2, t1, t2, dftemp, nb)
fwrite(dfest, "../Data/Negative binomial day-period estimates.csv")
#dfest <- fread("../Data/Negative binomial day-period estimates.csv")
rm(df, dfest); gc()






#####################################################################################################-
### Regression: Neg binomial estimates between t and t-1 (using Dec/22 as starting treat window) ####
#####################################################################################################-
##### Using data from 01-19-year to 01-18-year+2 (replicate authors sample) ####-
#Import data (all models exclude Central Park precinct)
df <- fread("../Data/Crime day-year-precinct data 2006-2019.csv") %>%
  mutate(date = mdy(paste(month, day, year, sep = "-"))) %>%
  filter(precinct != 22) 
#create indicator variables for series windows
for(X in 2006:2019){
  date1 <- mdy(paste0("01-19-", X)) 
  date2 <- mdy(paste0("01-18-", X+1)) 
  df[,paste0("s", X)] <- ifelse(df$date >= date1 & df$date <= date2, 1, 0)
} 
rm(date1, date2, X)
#create indicator variables for treatment window
for(X in 2006:2019){
  date1 <- mdy(paste0("12-22-", X)) 
  date2 <- mdy(paste0("01-18-", X+1)) 
  df[,paste0("t", X)] <- ifelse(df$date >= date1 & df$date <= date2, 1, 0)
} 
rm(date1, date2, X)
#Estimate the negative binomial regression by year-pairs (t vs t-1)
vnames <- df %>% 
  select(arrest_patrol, arrest_drugs, contains("major"), summons) %>% 
  names()
dfest <- data.frame()
#First loop goes over the outcome variable, the second one over the years-pairs
for(W in vnames){
  for(X in 2006:2017){
    #Create indicator variables
    s1 <- paste0("s", X); s2 <- paste0("s", X+1) 
    t1 <- paste0("t", X); t2 <- paste0("t", X+1) 
    #Subset data
    dftemp <- df %>%
      filter(!!sym(s1) == 1 | !!sym(s2) == 1) %>%
      mutate(series = ifelse(!!sym(s2) == 1, 1, 0), 
             treat = ifelse(!!sym(t1) == 1, 1, 0), 
             treat = ifelse(!!sym(t2) == 1, 1, treat)) 
    #Estimate regression (with and without precinct-fixed effects)
    for(Y in c("precinct.fe", "no.fe")){
      if(Y == "precinct.fe"){
        f <- as.formula(paste0(W, " ~ series*treat + snow + precipitation + temperature | precinct"))
      } else if(Y == "no.fe"){
        f <- as.formula(paste0(W, " ~ series*treat + snow + precipitation + temperature"))
      } 
      nb <- fenegbin(f, data = dftemp, cluster = ~ precinct)
      dfest <- dfest %>% 
        bind_rows(data.frame("outcome" = W, 
                             "period.treat" = paste0("dec", substr(X+1,3,4), "-jan", substr(X+2,3,4)),
                             "fe" = Y,
                             "irr" = exp(nb$coeftable["series:treat","Estimate"]), 
                             "se" = (exp(nb$coefficients)*sqrt(diag(vcov(nb))))["series:treat"], 
                             "ci.low" = exp(nb$coefficients - 1.96*sqrt(diag(vcov(nb))))["series:treat"],
                             "ci.up" = exp(nb$coefficients + 1.96*sqrt(diag(vcov(nb))))["series:treat"]))
    }
  }
  print(paste0("Ready: ", W))
}
rm(W, X, Y, vnames, f, s1, s2, t1, t2, dftemp, nb)
fwrite(dfest, "../Data/Negative binomial day-period estimates robustness.csv")
#dfest <- fread("../Data/Negative binomial day-period estimates robustness.csv")
rm(df, dfest); gc()







########################################################################################-
### Regression: Negative binomial estimates between t and t-1 (census tract level) #####
########################################################################################-
##### Using data from 01-19-year to 01-18-year+2 (replicate authors sample) ####-
#Import data (all models exclude Central Park precinct)
#It takes several hours to run
df <- fread("../Data/Crime day-year-tract data 2006-2019.csv", integer64 = "character") %>%
  mutate(date = mdy(paste(month, day, year, sep = "-"))) %>%
  filter(tractid != "36061014300") 
#create indicator variables for series windows
for(X in 2006:2019){
  date1 <- mdy(paste0("01-19-", X)) 
  date2 <- mdy(paste0("01-18-", X+1)) 
  df[,paste0("s", X)] <- ifelse(df$date >= date1 & df$date <= date2, 1, 0)
} 
rm(date1, date2, X)
#create indicator variables for treatment window
for(X in 2006:2019){
  date1 <- mdy(paste0("12-01-", X)) 
  date2 <- mdy(paste0("01-18-", X+1)) 
  df[,paste0("t", X)] <- ifelse(df$date >= date1 & df$date <= date2, 1, 0)
} 
rm(date1, date2, X)
#Estimate the negative binomial regression by year-pairs (t vs t-1)
vnames <- df %>% 
  select(arrest_patrol, arrest_drugs, contains("major"), summons) %>% 
  names()
dfest <- data.frame()
#First loop goes over the outcome variable, the second one over the years-pairs
for(W in vnames){
  for(X in 2006:2017){
    #Create indicator variables
    s1 <- paste0("s", X); s2 <- paste0("s", X+1) 
    t1 <- paste0("t", X); t2 <- paste0("t", X+1) 
    #Subset data
    dftemp <- df %>%
      filter(!!sym(s1) == 1 | !!sym(s2) == 1) %>%
      mutate(series = ifelse(!!sym(s2) == 1, 1, 0), 
             treat = ifelse(!!sym(t1) == 1, 1, 0), 
             treat = ifelse(!!sym(t2) == 1, 1, treat)) 
    #Estimate regression (with and without tract-fixed effects)
    for(Y in c("tractid.fe", "no.fe")){
      if(Y == "tractid.fe"){
        f <- as.formula(paste0(W, " ~ series*treat + snow + precipitation + temperature | tractid"))
      } else if(Y == "no.fe"){
        f <- as.formula(paste0(W, " ~ series*treat + snow + precipitation + temperature"))
      } 
      nb <- fenegbin(f, data = dftemp, cluster = ~ tractid)
      dfest <- dfest %>% 
        bind_rows(data.frame("outcome" = W, 
                             "period.treat" = paste0("dec", substr(X+1,3,4), "-jan", substr(X+2,3,4)),
                             "fe" = Y,
                             "irr" = exp(nb$coeftable["series:treat","Estimate"]), 
                             "se" = (exp(nb$coefficients)*sqrt(diag(vcov(nb))))["series:treat"], 
                             "ci.low" = exp(nb$coefficients - 1.96*sqrt(diag(vcov(nb))))["series:treat"],
                             "ci.up" = exp(nb$coefficients + 1.96*sqrt(diag(vcov(nb))))["series:treat"]))
    }
  }
  print(paste0("Ready: ", W))
}
rm(W, X, Y, vnames, f, s1, s2, t1, t2, dftemp, nb)
fwrite(dfest, "../Data/Negative binomial day-period estimates tract level.csv")
#dfest <- fread("../Data/Negative binomial day-period estimates tract level.csv")
rm(df, dfest); gc()






########################################################################################-
### Regression: Negative binomial estimates between t and t-1 (two way clustering) #####
########################################################################################-
##### Using data from 01-19-year to 01-18-year+2 (replicate authors sample) ####-
#Import data (all models exclude Central Park precinct)
df <- fread("../Data/Crime day-year-precinct data 2006-2019.csv") %>%
  mutate(date = mdy(paste(month, day, year, sep = "-"))) %>%
  filter(precinct != 22) 
#create indicator variables for series windows
for(X in 2006:2019){
  date1 <- mdy(paste0("01-19-", X)) 
  date2 <- mdy(paste0("01-18-", X+1)) 
  df[,paste0("s", X)] <- ifelse(df$date >= date1 & df$date <= date2, 1, 0)
} 
rm(date1, date2, X)
#create indicator variables for treatment window
for(X in 2006:2019){
  date1 <- mdy(paste0("12-01-", X)) 
  date2 <- mdy(paste0("01-18-", X+1)) 
  df[,paste0("t", X)] <- ifelse(df$date >= date1 & df$date <= date2, 1, 0)
} 
rm(date1, date2, X)
#Estimate the negative binomial regression by year-pairs (t vs t-1)
vnames <- df %>% 
  select(summons, crime_major) %>% 
  names()
dfest <- data.frame()
#First loop goes over the outcome variable, the second one over the years-pairs
for(W in vnames){
  for(X in 2006:2017){
    #Create indicator variables
    s1 <- paste0("s", X); s2 <- paste0("s", X+1) 
    t1 <- paste0("t", X); t2 <- paste0("t", X+1) 
    #Subset data
    dftemp <- df %>%
      filter(!!sym(s1) == 1 | !!sym(s2) == 1) %>%
      mutate(series = ifelse(!!sym(s2) == 1, 1, 0), 
             treat = ifelse(!!sym(t1) == 1, 1, 0), 
             treat = ifelse(!!sym(t2) == 1, 1, treat)) 
    #Estimate regression (with and without precinct-fixed effects)
    f <- as.formula(paste0(W, " ~ series*treat + snow + precipitation + temperature"))
    nb <- fenegbin(f, data = dftemp, cluster = ~ precinct + year)
    dfest <- dfest %>% 
      bind_rows(data.frame("outcome" = W, 
                           "period.treat" = paste0("dec", substr(X+1,3,4), "-jan", substr(X+2,3,4)),
                           "irr" = exp(nb$coeftable["series:treat","Estimate"]), 
                           "se" = (exp(nb$coefficients)*sqrt(diag(vcov(nb))))["series:treat"], 
                           "tvalue" = nb$coeftable["series:treat","t value"],
                           "pvalue" = nb$coeftable["series:treat","Pr(>|t|)"],
                           "ci.low" = exp(nb$coefficients - 1.96*sqrt(diag(vcov(nb))))["series:treat"],
                           "ci.up" = exp(nb$coefficients + 1.96*sqrt(diag(vcov(nb))))["series:treat"]))
  }
  print(paste0("Ready: ", W))
}
rm(W, X, vnames, f, s1, s2, t1, t2, dftemp, nb)
fwrite(dfest, "../Data/Negative binomial day-period estimates twoway.csv")
#dfest <- fread("../Data/Negative binomial day-period estimates twoway.csv")
rm(df, dfest)






#############################################################################################################-
### Regression: Neg binomial Event Study for main regression and high vs low exposure (2010-2015 period) ####
#############################################################################################################-
#Using data from 01-19-2010 to 01-18-2015 (longer pre-intervention time period but excluding financial crisis)
#Import data (all models exclude Central Park precinct)
df <- fread("../Data/Crime day-year-precinct data 2006-2019.csv") %>%
  filter(precinct != 22) %>%
  mutate(date = mdy(paste(month, day, year, sep = "-")), 
         week = isoweek(date))
#percentage difference between Dec/01/2014-Jan/18/2015 and Dec/01/2013-Jan/18/2014 
dftemp <- df %>%
  filter(year %in% 2013:2015) %>%
  mutate(period = ifelse(year == 2013 & month == 12, 2013, 0), 
         period = ifelse(year == 2014 & month == 1 & day %in% 1:18, 2013, period), 
         period = ifelse(year == 2014 & month == 12, 2014, period), 
         period = ifelse(year == 2015 & month == 1 & day %in% 1:18, 2014, period)) %>%
  select(period, precinct, crime_major, summons) %>%
  filter(period %in% 2013:2014) %>%
  group_by(period, precinct) %>%
  summarise_all(list(sum)) %>%
  as.data.frame() %>%
  arrange(precinct) %>%
  group_by(precinct) %>%
  mutate_at(vars(crime_major, summons), function(x) (x - lag(x))/lag(x)) %>%
  as.data.frame() %>%
  filter(period == 2014)
vprecincts <- dftemp %>%
  filter(summons < median(dftemp$summons)) %>%
  pull(precinct)
#Create indicators variables
df <- df %>%
  filter(date >= mdy("01-19-2010") & date <= mdy("01-18-2015")) %>%
  mutate(series = ifelse(date >= mdy("01-19-2014") & date <= mdy("01-18-2015"), 1, 0), 
         treat = ifelse(month==12 | (month==1 & day <= 18), 1, 0), 
         high = ifelse(precinct %in% vprecincts, 1, 0)) %>%
  mutate(series_treat = series*treat,
         series_high = series*high,
         treat_high = treat*high,
         series_treat_high = series*treat*high) %>%
  select(year, month, day, week, precinct, summons, crime_major, temperature:series_treat_high)
rm(dftemp, vprecincts)
#df %>% select(year, week, precinct, series, high) %>% View()
#Create variables for treatment-time dummy variables
df <- df %>%
  mutate(relweek = ifelse(week %in% 4:52, week - 49, week + 52 - 49))
dftemp <- as.data.frame(with(df, i(relweek, series, keep = c(-45:-2, 0:6))))
names(dftemp) <- paste0("tau", str_replace(c(-45:-2, 0:6), "-", "_"))
df <- df %>% 
  bind_cols(dftemp) %>%
  as.data.frame()
rm(dftemp)
#df %>% filter(precinct==1) %>% select(year, week, relweek, series, starts_with("tau")) %>% View()
#Create variables for treatment-time-exposure dummy variables
dftemp <- as.data.frame(with(df, i(relweek, series_high, keep = c(-45:-2, 0:6))))
names(dftemp) <- paste0("hightau", str_replace(c(-45:-2, 0:6), "-", "_"))
df <- df %>% 
  bind_cols(dftemp) %>%
  as.data.frame()
rm(dftemp)
#df %>% filter(precinct==1) %>% select(year, week, relweek, series, starts_with("high")) %>% View()

#Confirm main specification before running additional ones
f1 <- as.formula("crime_major ~ series*treat + snow + precipitation + temperature")
nb <- fenegbin(f1, data = df, cluster = ~ precinct)
exp(nb$coeftable["series:treat","Estimate"]) #close to 0.92 as it uses more pre-intervention periods

#Run event study of high vs low exposure neighborhoods
dfest <- data.frame()
for(X in c("crime_major")){
  f1 <- as.formula(paste0(X, "~ series + high + series_high + snow + precipitation + temperature + treat +", 
                          paste0("tau", str_replace(c(-45:-2, 0:6), "-", "_"), collapse = " + "), " + ",
                          paste0("hightau", str_replace(c(-45:-2, 0:6), "-", "_"), collapse = " + ")))
  nb <- fenegbin(f1, data = df, cluster = ~ precinct)
  dfest <- dfest %>%
    bind_rows(data.frame("outcome" = X, 
                         "variable" = grep("^hightau*", names(nb$coefficients), value = TRUE),
                         "week" = c(-45:-2, 0:6),
                         "irr" = exp(nb$coeftable[grep("^hightau*", names(nb$coefficients), value = TRUE),"Estimate"]), 
                         "se" = (exp(nb$coefficients)*sqrt(diag(vcov(nb))))[grep("^hightau*", names(nb$coefficients), value = TRUE)], 
                         "ci.low" = exp(nb$coefficients - 1.96*sqrt(diag(vcov(nb))))[grep("^hightau*", names(nb$coefficients), value = TRUE)],
                         "ci.up" = exp(nb$coefficients + 1.96*sqrt(diag(vcov(nb))))[grep("^hightau*", names(nb$coefficients), value = TRUE)]))
}
fwrite(dfest, "../Data/Negative binomial week-period estimates event study highlow longperiod.csv")
#dfest <- fread("../Data/Negative binomial week-period estimates event study highlow longperiod.csv")
rm(df, dfest, nb, f1, X); gc()






###################################################################################-
### Plot: time series main outcomes at the day-year-city level, 2006-2019 #########
###################################################################################-
#Aggregate at the day-year-city level (exclude precint 22 as it is Central Park)
df <- fread("../Data/Crime day-year-precinct data 2006-2019.csv")
dfcity <- df %>%
  filter(precinct != 22) %>%
  select(-precinct) %>%
  group_by(year, month, day) %>%
  summarise_all(list(sum)) %>%
  as.data.frame() %>%
  mutate(date = mdy(paste(month, day, year, sep = "-")))
for(X in 2006:2019){
  date1 <- mdy(paste0("06-01-", X)) # june 1st of each year
  date2 <- mdy(paste0("06-01-", X+1)) # june 1st of each year
  dfcity[,paste0("i", X)] <-ifelse(dfcity$date>=date1 & dfcity$date<date2, X, 0)
} 
vnames <- dfcity %>% 
  select(arrest_nonmajor, arrest_major, starts_with("crime"), summons, stopfrisk) %>%
  names(.)
plottheme <- theme(
  axis.text.x = element_text(size = 16),
  axis.text.y = element_text(size = 16),
  axis.title.x = element_text(size = 18),
  axis.title.y = element_text(size = 18),
  strip.text.x = element_text(size = 15),
  strip.background = element_rect(fill = "grey85", color = "black"),
  panel.background = element_blank(), axis.line = element_line(colour = "black"),
  legend.key = element_rect(colour = "transparent", fill = "transparent"),
  legend.key.width = unit(1.5, "cm"),
  legend.text = element_text(size = 20),
  legend.title = element_text(size = 20),
  legend.position = "none",
  rect = element_rect(fill = "transparent"))

### Time series 2014/2015 (Oct/2014 to Feb/2015)
dftemp <- dfcity %>%
  filter(date >= mdy("06-01-2006") | date <= mdy("06-01-2019")) %>%
  mutate(period = rowSums(select(., i2006:i2019))) %>%
  select(year:arrest_nonmajor, arrest_major, starts_with("crime"), summons, stopfrisk, period, date) %>%
  mutate(yday = yday(date)) %>%
  pivot_longer(cols = arrest_nonmajor:stopfrisk, names_to = "outcome", values_to = "count") %>%
  mutate(yday2 = ifelse(year %in% c(2008,2012,2016) & yday >=153, yday - 366, yday),
         yday2 = ifelse(year %in% c(2006,2007,2009:2011,2013:2015,2017:2019) & yday >= 152, yday - 365, yday2))
dftemp2 <- dftemp %>% 
  filter(period == 2014) %>%
  filter(yday %in% c(1:60, 274:366))
X <- "summons"
ylim1 <- 1510
yvar <- 250
p <- ggplot(dftemp2 %>% filter(outcome == X)) +
  geom_line(aes(x = yday2, y = count, group = period)) +
  geom_point(aes(x = yday2, y = count, group = period), size = 2) +
  geom_vline(xintercept = c(-30, 18), linetype = "dashed", color = "darkblue", size = 1.2) +
  geom_vline(xintercept = c(-10), linetype = "dotted", color = "darkred", size = 1.2) +
  annotate(geom = "text", x = -33, y = 1350, label = "December 1st", angle = 90, size = 5) +
  annotate(geom = "text", x = -13, y = 1350, label = "December 22nd", angle = 90, size = 5) +
  annotate(geom = "text", x = 15, y = 1350, label = "January 18th", angle = 90, size = 5) +
  labs(color = "", y = "incidents", x = NULL) +
  scale_x_continuous(breaks = c(-91, -60, -30, 1, 32), labels = c("Oct/14", "Nov/14", "Dec/14", "Jan/15", "Feb/15")) +
  scale_y_continuous(breaks = seq(0, ylim1, yvar), limits = c(0, ylim1)) +
  plottheme
ggsave("../Tables-Figures/Figure1B.jpg", plot = p, width = 9, height = 6, dpi = 300)  
  

### Time series Oct/2013 to Jan/2016
dftemp <- dfcity %>%
  filter(date >= mdy("10-01-2013") & date <= mdy("03-01-2016"))
X <- "summons"
ylim1 <- 2250
yvar <- 500
p <- ggplot(dftemp, aes_string(x = "date", y = X)) +
  geom_line(color = "grey40") +
  geom_point(size = 1.5) +
  geom_vline(xintercept = c(mdy("12-01-2014"), mdy("01-18-2015")), linetype = "dashed", color = "darkblue", size = 1) +
  geom_vline(xintercept = c(mdy("12-22-2014")), linetype = "dotted", color = "darkred", size = 1) +
  annotate(geom = "text", x = mdy("11-20-2014"), y = 1850, label = "December 1st", angle = 90, size = 4) +
  annotate(geom = "text", x = mdy("12-10-2014"), y = 1850, label = "December 22nd", angle = 90, size = 4) +
  annotate(geom = "text", x = mdy("01-05-2015"), y = 1850, label = "January 18th", angle = 90, size = 4) +
  labs(color = "", y = "incidents", x = NULL) +
  scale_x_continuous(breaks = c(mdy("01-01-2014"), mdy("07-01-2014"), mdy("01-01-2015"), mdy("07-01-2015"),
                                mdy("01-01-2016")), 
                     labels = c("Jan/2014", "July/2014", "Jan/2015", "July/2015", "Jan/2016")) +
  scale_y_continuous(breaks = seq(0, ylim1, yvar), limits = c(0, ylim1)) +
  plottheme
ggsave("../Tables-Figures/Figure1A.jpg", plot = p, width = 9, height = 5,  dpi = 300)  
rm(df, dfcity, dftemp, dftemp2, p, plottheme, date1, date2, vnames, X, ylim1, yvar)





#####################################################################################-
### Plot: scatterplot arrest vs crimes period-precinct changes between t and t-1 ####
#####################################################################################-
#plot theme
plottheme <- theme(
  axis.text.x = element_text(size = 20),
  axis.text.y = element_text(size = 20),
  axis.title.x = element_text(size = 20),
  axis.title.y = element_text(size = 20),
  strip.text.x = element_text(size = 15),
  strip.background = element_rect(fill = "grey85", color = "black"),
  panel.background = element_blank(), axis.line = element_line(colour = "black"),
  legend.key = element_rect(colour = "transparent", fill = "transparent"),
  legend.key.width = unit(1.5, "cm"),
  legend.text = element_text(size = 20),
  legend.title = element_text(size = 20),
  legend.position = "none",
  rect = element_rect(fill = "transparent"))

### Percent changes ######-
#Aggregate at the group-precinct level (exclude precint 22 as it is Central Park)
#group is the 12/01/year to 01/18/year+1 period
df <- fread("../Data/Crime day-year-precinct data 2006-2019.csv") %>%
  mutate(date = mdy(paste(month, day, year, sep = "-"))) %>%
  filter(precinct != 22)
for(X in 2006:2019){
  date1 <- mdy(paste0("12-01-", X)) # december 1st of each year
  date2 <- mdy(paste0("01-18-", X+1)) # january 18th of each year
  df[,paste0("i", X)] <-ifelse(df$date >= date1 & df$date <= date2, X, 0)
} 
rm(date1, date2, X)
#percentage difference by period and precinct
dftemp <- df %>%
  mutate(period = rowSums(select(., i2006:i2019))) %>%
  select(period, precinct, arrest_nonmajor, arrest_major, arrest_patrol, arrest_drugs, starts_with("crime"), summons, stopfrisk) %>%
  filter(period %in% 2006:2018) %>%
  group_by(period, precinct) %>%
  summarise_all(list(sum)) %>%
  as.data.frame() %>%
  arrange(precinct) %>%
  group_by(precinct) %>%
  mutate_at(vars(arrest_nonmajor:stopfrisk), function(x) (x - lag(x))/lag(x)) %>%
  as.data.frame() %>%
  filter(period == 2014) %>%
  mutate(period2 = paste0("dec",substr(period,3,4),"-jan",substr(period+1,3,4)," / dec", 
                          substr(period-1,3,4),"-jan", substr(period,3,4)))

#Plot
X <- "summons"
Y <- "crime_major"
f1 <- as.formula(paste0(Y, "~", X))
lm1 <- lm(f1, data = dftemp)
alpha <- round(unname(lm1$coefficients["(Intercept)"]), 3)
beta <- round(unname(lm1$coefficients[X]), 3)
beta <- ifelse(beta >= 0, paste0(" + ", beta), paste0(" - ", abs(beta)))
r2 <- round(summary(lm1)$r.squared, 5)
vform1 <- paste0("y = ", alpha, beta, "*x")
vform2 <- paste0("R^2 == ", r2)
xlab <- "Precinct-yearly criminal summons change (0.1 = 10%)"
p <- ggplot(dftemp, aes_string(x = X, y = Y)) +
  geom_point(alpha = 0.8, shape = 1, color = "black", size = 5) +
  geom_smooth(method = "lm", alpha = 0.2, color = "black") +
  geom_vline(xintercept = 0, linetype = "dashed", color = "grey50") +
  geom_hline(yintercept = 0, linetype = "dashed", color = "grey50") +
  labs(x = xlab, y = "Precinct-yearly crime change (0.1=10%)") +
  annotate(geom = "text", x = -0.15, y = 0.60, label = vform1, size = 8) +
  annotate(geom = "text", x = -0.15, y = 0.50, label = vform2, parse = TRUE, size = 8) +
  ylim(-0.7, 0.7) +
  plottheme  
ggsave("../Tables-Figures/Figure3B.jpg", plot = p, width = 9, height = 7,  dpi = 300)  

#Plot
X <- "arrest_nonmajor"
Y <- "crime_nonmajor"
f1 <- as.formula(paste0(Y, "~", X))
lm1 <- lm(f1, data = dftemp)
alpha <- round(unname(lm1$coefficients["(Intercept)"]), 3)
beta <- round(unname(lm1$coefficients[X]), 3)
beta <- ifelse(beta >= 0, paste0(" + ", beta), paste0(" - ", abs(beta)))
r2 <- round(summary(lm1)$r.squared, 5)
vform1 <- paste0("y = ", alpha, beta, "*x")
vform2 <- paste0("R^2 == ", r2)
xlab <- "Precinct-yearly non-major crime arrests change (0.1=10%)"
p <- ggplot(dftemp, aes_string(x = X, y = Y)) +
  geom_point(alpha = 0.8, shape = 1, color = "black", size = 5) +
  geom_smooth(method = "lm", alpha = 0.2, color = "black") +
  geom_vline(xintercept = 0, linetype = "dashed", color = "grey50") +
  geom_hline(yintercept = 0, linetype = "dashed", color = "grey50") +
  labs(x = xlab, y = "Precinct-yearly crime change (0.1=10%)") +
  annotate(geom = "text", x = -0.15, y = 0.60, label = vform1, size = 8) +
  annotate(geom = "text", x = -0.15, y = 0.50, label = vform2, parse = TRUE, size = 8) +
  ylim(-0.7, 0.7) +
  plottheme  
ggsave("../Tables-Figures/FigureA5A.jpg", plot = p, width = 9, height = 7,  dpi = 300)  

#Plot
X <- "arrest_nonmajor"
Y <- "crime_major"
f1 <- as.formula(paste0(Y, "~", X))
lm1 <- lm(f1, data = dftemp)
alpha <- round(unname(lm1$coefficients["(Intercept)"]), 3)
beta <- round(unname(lm1$coefficients[X]), 3)
beta <- ifelse(beta >= 0, paste0(" + ", beta), paste0(" - ", abs(beta)))
r2 <- round(summary(lm1)$r.squared, 5)
vform1 <- paste0("y = ", alpha, beta, "*x")
vform2 <- paste0("R^2 == ", r2)
xlab <- "Precinct-yearly non-major crime arrests change (0.1=10%)"
p <- ggplot(dftemp, aes_string(x = X, y = Y)) +
  geom_point(alpha = 0.8, shape = 1, color = "black", size = 5) +
  geom_smooth(method = "lm", alpha = 0.2, color = "black") +
  geom_vline(xintercept = 0, linetype = "dashed", color = "grey50") +
  geom_hline(yintercept = 0, linetype = "dashed", color = "grey50") +
  labs(x = xlab, y = "Precinct-yearly crime change (0.1=10%)") +
  annotate(geom = "text", x = -0.15, y = 0.60, label = vform1, size = 8) +
  annotate(geom = "text", x = -0.15, y = 0.50, label = vform2, parse = TRUE, size = 8) +
  ylim(-0.7, 0.7) +
  plottheme  
ggsave("../Tables-Figures/FigureA5B.jpg", plot = p, width = 9, height = 7,  dpi = 300)  

#Plot
X <- "arrest_nonmajor"
Y <- "crime_major_in"
f1 <- as.formula(paste0(Y, "~", X))
lm1 <- lm(f1, data = dftemp)
alpha <- round(unname(lm1$coefficients["(Intercept)"]), 3)
beta <- round(unname(lm1$coefficients[X]), 3)
beta <- ifelse(beta >= 0, paste0(" + ", beta), paste0(" - ", abs(beta)))
r2 <- round(summary(lm1)$r.squared, 5)
vform1 <- paste0("y = ", alpha, beta, "*x")
vform2 <- paste0("R^2 == ", r2)
xlab <- "Precinct-yearly non-major crime arrests change (0.1=10%)"
p <- ggplot(dftemp, aes_string(x = X, y = Y)) +
  geom_point(alpha = 0.8, shape = 1, color = "black", size = 5) +
  geom_smooth(method = "lm", alpha = 0.2, color = "black") +
  geom_vline(xintercept = 0, linetype = "dashed", color = "grey50") +
  geom_hline(yintercept = 0, linetype = "dashed", color = "grey50") +
  labs(x = xlab, y = "Precinct-yearly crime change (0.1=10%)") +
  annotate(geom = "text", x = -0.15, y = 0.60, label = vform1, size = 8) +
  annotate(geom = "text", x = -0.15, y = 0.50, label = vform2, parse = TRUE, size = 8) +
  ylim(-0.7, 0.7) +
  plottheme  
ggsave("../Tables-Figures/FigureA5C.jpg", plot = p, width = 9, height = 7,  dpi = 300) 

#Plot
X <- "arrest_nonmajor"
Y <- "crime_major_out"
f1 <- as.formula(paste0(Y, "~", X))
lm1 <- lm(f1, data = dftemp)
alpha <- round(unname(lm1$coefficients["(Intercept)"]), 3)
beta <- round(unname(lm1$coefficients[X]), 3)
beta <- ifelse(beta >= 0, paste0(" + ", beta), paste0(" - ", abs(beta)))
r2 <- round(summary(lm1)$r.squared, 5)
vform1 <- paste0("y = ", alpha, beta, "*x")
vform2 <- paste0("R^2 == ", r2)
xlab <- "Precinct-yearly non-major crime arrests change (0.1=10%)"
p <- ggplot(dftemp, aes_string(x = X, y = Y)) +
  geom_point(alpha = 0.8, shape = 1, color = "black", size = 5) +
  geom_smooth(method = "lm", alpha = 0.2, color = "black") +
  geom_vline(xintercept = 0, linetype = "dashed", color = "grey50") +
  geom_hline(yintercept = 0, linetype = "dashed", color = "grey50") +
  labs(x = xlab, y = "Precinct-yearly crime change (0.1=10%)") +
  annotate(geom = "text", x = -0.15, y = 0.60, label = vform1, size = 8) +
  annotate(geom = "text", x = -0.15, y = 0.50, label = vform2, parse = TRUE, size = 8) +
  ylim(-0.7, 0.7) +
  plottheme  
ggsave("../Tables-Figures/FigureA5D.jpg", plot = p, width = 9, height = 7,  dpi = 300)  

#Plot
X <- "summons"
Y <- "crime_major_in"
f1 <- as.formula(paste0(Y, "~", X))
lm1 <- lm(f1, data = dftemp)
alpha <- round(unname(lm1$coefficients["(Intercept)"]), 3)
beta <- round(unname(lm1$coefficients[X]), 3)
beta <- ifelse(beta >= 0, paste0(" + ", beta), paste0(" - ", abs(beta)))
r2 <- round(summary(lm1)$r.squared, 5)
vform1 <- paste0("y = ", alpha, beta, "*x")
vform2 <- paste0("R^2 == ", r2)
xlab <- "Precinct-yearly criminal summons change (0.1 = 10%)"
p <- ggplot(dftemp, aes_string(x = X, y = Y)) +
  geom_point(alpha = 0.8, shape = 1, color = "black", size = 5) +
  geom_smooth(method = "lm", alpha = 0.2, color = "black") +
  geom_vline(xintercept = 0, linetype = "dashed", color = "grey50") +
  geom_hline(yintercept = 0, linetype = "dashed", color = "grey50") +
  labs(x = xlab, y = "Precinct-yearly crime change (0.1=10%)") +
  annotate(geom = "text", x = -0.15, y = 0.60, label = vform1, size = 8) +
  annotate(geom = "text", x = -0.15, y = 0.50, label = vform2, parse = TRUE, size = 8) +
  ylim(-0.7, 0.7) +
  plottheme  
ggsave("../Tables-Figures/FigureA5E.jpg", plot = p, width = 9, height = 7,  dpi = 300)  

#Plot
X <- "summons"
Y <- "crime_major_out"
f1 <- as.formula(paste0(Y, "~", X))
lm1 <- lm(f1, data = dftemp)
alpha <- round(unname(lm1$coefficients["(Intercept)"]), 3)
beta <- round(unname(lm1$coefficients[X]), 3)
beta <- ifelse(beta >= 0, paste0(" + ", beta), paste0(" - ", abs(beta)))
r2 <- round(summary(lm1)$r.squared, 5)
vform1 <- paste0("y = ", alpha, beta, "*x")
vform2 <- paste0("R^2 == ", r2)
xlab <- "Precinct-yearly criminal summons change (0.1 = 10%)"
p <- ggplot(dftemp, aes_string(x = X, y = Y)) +
  geom_point(alpha = 0.8, shape = 1, color = "black", size = 5) +
  geom_smooth(method = "lm", alpha = 0.2, color = "black") +
  geom_vline(xintercept = 0, linetype = "dashed", color = "grey50") +
  geom_hline(yintercept = 0, linetype = "dashed", color = "grey50") +
  labs(x = xlab, y = "Precinct-yearly crime change (0.1=10%)") +
  annotate(geom = "text", x = -0.15, y = 0.60, label = vform1, size = 8) +
  annotate(geom = "text", x = -0.15, y = 0.50, label = vform2, parse = TRUE, size = 8) +
  ylim(-0.7, 0.7) +
  plottheme  
ggsave("../Tables-Figures/FigureA5F.jpg", plot = p, width = 9, height = 7,  dpi = 300)  
rm(df, dftemp, lm1, p, plottheme, alpha, beta, f1, r2, vform1, vform2, X, xlab, Y); gc()





#########################################################################################################################-
### Plot: scatterplot arrest vs crimes period-precinct changes between t and t-1 (changes of rates per 1K residents) ####
#########################################################################################################################-
#plot theme
plottheme <- theme(
  axis.text.x = element_text(size = 20),
  axis.text.y = element_text(size = 20),
  axis.title.x = element_text(size = 20),
  axis.title.y = element_text(size = 20),
  strip.text.x = element_text(size = 15),
  strip.background = element_rect(fill = "grey85", color = "black"),
  panel.background = element_blank(), axis.line = element_line(colour = "black"),
  legend.key = element_rect(colour = "transparent", fill = "transparent"),
  legend.key.width = unit(1.5, "cm"),
  legend.text = element_text(size = 20),
  legend.title = element_text(size = 20),
  legend.position = "none",
  rect = element_rect(fill = "transparent"))

### Percent changes ######-
#Aggregate at the group-precinct level (exclude precinct 22 as it is Central Park)
#group is the 12/01/year to 01/18/year+1 period
df <- fread("../Data/Crime day-year-precinct data 2006-2019.csv") %>%
  mutate(date = mdy(paste(month, day, year, sep = "-"))) %>%
  filter(precinct != 22) %>%
  mutate_at(vars(arrest_nonmajor:stopfrisk), funs(./population*1000))
for(X in 2006:2019){
  date1 <- mdy(paste0("12-01-", X)) # december 1st of each year
  date2 <- mdy(paste0("01-18-", X+1)) # january 18th of each year
  df[,paste0("i", X)] <-ifelse(df$date >= date1 & df$date <= date2, X, 0)
} 
rm(date1, date2, X)
#rate difference by period and precinct
dftemp <- df %>%
  mutate(period = rowSums(select(., i2006:i2019))) %>%
  select(period, precinct, arrest_nonmajor, arrest_major, starts_with("crime"), summons, stopfrisk) %>%
  filter(period %in% 2006:2018) %>%
  group_by(period, precinct) %>%
  summarise_all(list(sum)) %>%
  as.data.frame() %>%
  arrange(precinct) %>%
  group_by(precinct) %>%
  mutate_at(vars(arrest_nonmajor:stopfrisk), function(x) x - lag(x)) %>%
  as.data.frame() %>%
  filter(period == 2014) %>%
  mutate(period2 = paste0("dec",substr(period,3,4),"-jan",substr(period+1,3,4)," / dec", 
                          substr(period-1,3,4),"-jan", substr(period,3,4)))
 
#Plots (comparing complete sample and without outlier with annotations version 2)
X <- "summons"; Y <- "crime_major"
f1 <- as.formula(paste0(Y, "~", X))
lm1 <- lm(f1, data = dftemp)
lm2 <- lm(f1, data = dftemp %>% filter(precinct != 14))
alpha1 <- round(unname(lm1$coefficients["(Intercept)"]), 2)
alpha2 <- round(unname(lm2$coefficients["(Intercept)"]), 2)
beta1 <- round(unname(lm1$coefficients[X]), 2)
beta2 <- round(unname(lm2$coefficients[X]), 2)
beta1 <- ifelse(beta1 >= 0, paste0(" + ", beta1), paste0(" - ", abs(beta1)))
beta2 <- ifelse(beta2 >= 0, paste0(" + ", beta2), paste0(" - ", abs(beta2)))
r1 <- round(summary(lm1)$r.squared, 2)
r2 <- round(summary(lm2)$r.squared, 2)
vcor1 <- with(dftemp, cor.test(summons, crime_major))
vcor2 <- with(dftemp[dftemp$precinct!=14,], cor.test(summons, crime_major))
pvalue1 <- ifelse(vcor1$p.value < 0.01, "p < 0.01", paste0("p = ", round(vcor1$p.value, 2)))
pvalue2 <- ifelse(vcor2$p.value < 0.01, "p < 0.01", paste0("p = ", round(vcor2$p.value, 2)))
rho1 <- paste0("r = ", round(vcor1$estimate, 2)) 
rho2 <- paste0("r = ", round(vcor2$estimate, 2))
vform11 <- paste0("y = ", alpha1, beta1, "*x,")
vform21 <- paste0("R^2==", r1)
vform31 <- paste0(rho1, " (", pvalue1, ")")
vform12 <- paste0("y = ", alpha2, beta2, "*x,")
vform22 <- paste0("R^2==", r2)
vform32 <- paste0(rho2, " (", pvalue2, ")")
pred1 <- predict(lm1, data = dftemp, se = TRUE)
pred2 <- predict(lm2, data = dftemp %>% filter(precinct != 14), se = TRUE)
dfpred1 <- dftemp %>%
  mutate(pred = pred1$fit,
         upper = pred1$fit - qnorm(0.975)*pred1$se.fit,
         lower = pred1$fit + qnorm(0.975)*pred1$se.fit)
dfpred2 <- dftemp %>%
  filter(precinct != 14) %>%
  mutate(pred = pred2$fit,
         upper = pred2$fit - qnorm(0.975)*pred2$se.fit,
         lower = pred2$fit + qnorm(0.975)*pred2$se.fit)
xlab <- "Precinct-yearly criminal summons rate change"
p <- ggplot(dftemp, aes_string(x = X, y = Y)) +
  geom_point(alpha = 0.8, shape = 1, color = "black", size = 5) +
  geom_line(aes(y = pred), size = 1.3, color = "darkblue", data = dfpred1, linetype = "dashed") +
  geom_line(aes(y = pred), size = 1.3, color = "darkred", data = dfpred2) +
  geom_vline(xintercept = 0, linetype = "dashed", color = "grey50") +
  geom_hline(yintercept = 0, linetype = "dashed", color = "grey50") +
  labs(x = xlab, y = "Precinct-yearly crime rate change") +
  annotate(geom = "text", x = -39, y = -0.2, label = vform12, size = 8, color = "darkred") +
  annotate(geom = "text", x = -26, y = -0.18, label = vform22, parse = TRUE, size = 8, color = "darkred") +
  annotate(geom = "text", x = -39, y = -0.42, label = vform32, size = 8, color = "darkred") +
  annotate(geom = "text", x = -39, y = -0.8, label = vform11, size = 8, color = "darkblue") +
  annotate(geom = "text", x = -26, y = -0.78, label = vform21, parse = TRUE, size = 8, color = "darkblue") +
  annotate(geom = "text", x = -39, y = -1.02, label = vform31, size = 8, color = "darkblue") +
  annotate(geom = "text", x = -24, y = -2.6, label = "Midtown South Precinct\n(includes Times Square)", size = 6) +
  annotate(geom = "curve", x = -46, y = -3.18, xend = -29, yend = -2.85, 
           arrow = arrow(type = "closed", length = unit(0.3, "cm"))) +
  ylim(-3.4, 0.5) +
  plottheme
ggsave(paste0("../Tables-Figures/FigureA4.jpg"), plot = p, width = 9, height = 7,  dpi = 300) 

#precinct 14 is an outlier
#summons vs major crimes
with(dftemp, cor.test(summons, crime_major))
with(dftemp[dftemp$precinct!=14,], cor.test(summons, crime_major))
#summons vs non-major crimes
with(dftemp, cor.test(summons, crime_nonmajor))
with(dftemp[dftemp$precinct!=14,], cor.test(summons, crime_nonmajor))
#non-major crime arrests vs major crimes
with(dftemp, cor.test(arrest_nonmajor, crime_major))
with(dftemp[dftemp$precinct!=14,], cor.test(arrest_nonmajor, crime_major))
#non-major crime arrests vs nonmajor crimes
with(dftemp, cor.test(arrest_nonmajor, crime_nonmajor))
with(dftemp[dftemp$precinct!=14,], cor.test(arrest_nonmajor, crime_nonmajor))





#######################################################################################################-
### Plot: scatterplot arrest vs crimes period-tract changes between t and t-1 (census tract level) ####
#######################################################################################################-
#plot theme
plottheme <- theme(
  axis.text.x = element_text(size = 20),
  axis.text.y = element_text(size = 20),
  axis.title.x = element_text(size = 20),
  axis.title.y = element_text(size = 20),
  strip.text.x = element_text(size = 15),
  strip.background = element_rect(fill = "grey85", color = "black"),
  panel.background = element_blank(), axis.line = element_line(colour = "black"),
  legend.key = element_rect(colour = "transparent", fill = "transparent"),
  legend.key.width = unit(1.5, "cm"),
  legend.text = element_text(size = 20),
  legend.title = element_text(size = 20),
  legend.position = "none",
  rect = element_rect(fill = "transparent"))

### Percent changes ######-
#Aggregate at the group-tract level (exclude tract 36061014300 as it is Central Park)
#group is the 12/01/year to 01/18/year+1 period
df <- fread("../Data/Crime day-year-tract data 2006-2019.csv") %>%
  mutate(date = mdy(paste(month, day, year, sep = "-"))) %>%
  filter(tractid != "36061014300") 
for(X in 2006:2019){
  date1 <- mdy(paste0("12-01-", X)) # december 1st of each year
  date2 <- mdy(paste0("01-18-", X+1)) # january 18th of each year
  df[,paste0("i", X)] <-ifelse(df$date >= date1 & df$date <= date2, X, 0)
} 
rm(date1, date2, X)
#percentage difference by period and tract
dftemp <- df %>%
  mutate(period = rowSums(select(., i2006:i2019))) %>%
  select(period, tractid, arrest_nonmajor, arrest_major, starts_with("crime"), summons, stopfrisk) %>%
  filter(period %in% 2006:2018) %>%
  group_by(period, tractid) %>%
  summarise_all(list(sum)) %>%
  as.data.frame() %>%
  arrange(tractid) %>%
  group_by(tractid) %>%
  mutate_at(vars(arrest_nonmajor:stopfrisk), function(x) (x - lag(x))/lag(x)) %>%
  as.data.frame() %>%
  filter(period == 2014) %>%
  mutate(period2 = paste0("dec",substr(period,3,4),"-jan",substr(period+1,3,4)," / dec", 
                          substr(period-1,3,4),"-jan", substr(period,3,4)))

#Plot
X <- "summons"
Y <- "crime_major"
f1 <- as.formula(paste0(Y, "~", X))
lm1 <- lm(f1, data = dftemp %>% filter(is.finite(!!sym(X)) & is.finite(!!sym(Y))))
alpha <- round(unname(lm1$coefficients["(Intercept)"]), 3)
beta <- round(unname(lm1$coefficients[X]), 3)
beta <- ifelse(beta >= 0, paste0(" + ", beta), paste0(" - ", abs(beta)))
r2 <- round(summary(lm1)$r.squared, 4)
n <- scales::comma(nrow(lm1$model))
vform1 <- paste0("y = ", alpha, beta, "*x")
vform2 <- paste0("R^2 == ", r2)
vform3 <- paste0("N = ", n)
xlab <- "Tract-yearly criminal summons change (0.1=10%)"
p <- ggplot(dftemp, aes_string(x = X, y = Y)) +
  geom_point(alpha = 0.8, shape = 1, color = "grey50", size = 5) +
  geom_smooth(method = "lm", alpha = 0.2, color = "black") +
  geom_vline(xintercept = 0, linetype = "dashed", color = "grey50") +
  geom_hline(yintercept = 0, linetype = "dashed", color = "grey50") +
  labs(x = xlab, y = "Tract-yearly crime change (0.1=10%)") +
  annotate(geom = "text", x = 3, y = 4, label = vform1, size = 8) +
  annotate(geom = "text", x = 3, y = 3.5, label = vform2, parse = TRUE, size = 8) +
  annotate(geom = "text", x = 3, y = 3, label = vform3, size = 8) +
  coord_cartesian(xlim = c(-1, 5), ylim = c(-1, 5)) +
  plottheme  
ggsave("../Tables-Figures/FigureA9A.jpg", plot = p, width = 9, height = 7,  dpi = 300)  

#Plot
X <- "arrest_nonmajor"
Y <- "crime_nonmajor"
f1 <- as.formula(paste0(Y, "~", X))
lm1 <- lm(f1, data = dftemp %>% filter(is.finite(!!sym(X)) & is.finite(!!sym(Y))))
alpha <- round(unname(lm1$coefficients["(Intercept)"]), 3)
beta <- round(unname(lm1$coefficients[X]), 3)
beta <- ifelse(beta >= 0, paste0(" + ", beta), paste0(" - ", abs(beta)))
r2 <- round(summary(lm1)$r.squared, 4)
n <- scales::comma(nrow(lm1$model))
vform1 <- paste0("y = ", alpha, beta, "*x")
vform2 <- paste0("R^2 == ", r2)
vform3 <- paste0("N = ", n)
xlab <- "Tract-yearly non-major crime arrests change (0.1=10%)"
p <- ggplot(dftemp, aes_string(x = X, y = Y)) +
  geom_point(alpha = 0.8, shape = 1, color = "grey50", size = 5) +
  geom_smooth(method = "lm", alpha = 0.2, color = "black") +
  geom_vline(xintercept = 0, linetype = "dashed", color = "grey50") +
  geom_hline(yintercept = 0, linetype = "dashed", color = "grey50") +
  labs(x = xlab, y = "Tract-yearly crime change (0.1=10%)") +
  annotate(geom = "text", x = 3, y = 4, label = vform1, size = 8) +
  annotate(geom = "text", x = 3, y = 3.5, label = vform2, parse = TRUE, size = 8) +
  annotate(geom = "text", x = 3, y = 3, label = vform3, size = 8) +
  coord_cartesian(xlim = c(-1, 5), ylim = c(-1, 5)) +
  plottheme  
ggsave("../Tables-Figures/FigureA9B.jpg", plot = p, width = 9, height = 7,  dpi = 300)  
rm(df, dftemp, lm1, p, plottheme, alpha, beta, f1, n, r2, vform1, vform2, vform3, X, xlab, Y); gc()






####################################################################################################################-
### Plot: scatterplot arrest vs crimes period-precinct changes between t and t-1 (alternative treatment window) ####
####################################################################################################################-
#plot theme
plottheme <- theme(
  axis.text.x = element_text(size = 20),
  axis.text.y = element_text(size = 20),
  axis.title.x = element_text(size = 20),
  axis.title.y = element_text(size = 20),
  strip.text.x = element_text(size = 15),
  strip.background = element_rect(fill = "grey85", color = "black"),
  panel.background = element_blank(), axis.line = element_line(colour = "black"),
  legend.key = element_rect(colour = "transparent", fill = "transparent"),
  legend.key.width = unit(1.5, "cm"),
  legend.text = element_text(size = 20),
  legend.title = element_text(size = 20),
  legend.position = "none",
  rect = element_rect(fill = "transparent"))

### Percent changes ######-
#Aggregate at the group-precinct level (exclude precinct 22 as it is Central Park)
#group is the 12/01/year to 01/18/year+1 period
df <- fread("../Data/Crime day-year-precinct data 2006-2019.csv") %>%
  mutate(date = mdy(paste(month, day, year, sep = "-"))) %>%
  filter(precinct != 22)
for(X in 2006:2019){
  date1 <- mdy(paste0("12-22-", X)) # december 1st of each year
  date2 <- mdy(paste0("01-18-", X+1)) # january 18th of each year
  df[,paste0("i", X)] <-ifelse(df$date >= date1 & df$date <= date2, X, 0)
} 
rm(date1, date2, X)
#percentage difference by period and precinct
dftemp <- df %>%
  mutate(period = rowSums(select(., i2006:i2019))) %>%
  select(period, precinct, arrest_nonmajor, arrest_major, starts_with("crime"), summons, stopfrisk) %>%
  filter(period %in% 2006:2018) %>%
  group_by(period, precinct) %>%
  summarise_all(list(sum)) %>%
  as.data.frame() %>%
  arrange(precinct) %>%
  group_by(precinct) %>%
  mutate_at(vars(arrest_nonmajor:stopfrisk), function(x) (x - lag(x))/lag(x)) %>%
  as.data.frame() %>%
  filter(period == 2014) %>%
  mutate(period2 = paste0("dec",substr(period,3,4),"-jan",substr(period+1,3,4)," / dec", 
                          substr(period-1,3,4),"-jan", substr(period,3,4)))
#Plots
X <- "summons"
Y <- "crime_major"
f1 <- as.formula(paste0(Y, "~", X))
lm1 <- lm(f1, data = dftemp)
alpha <- round(unname(lm1$coefficients["(Intercept)"]), 3)
beta <- round(unname(lm1$coefficients[X]), 3)
beta <- ifelse(beta >= 0, paste0(" + ", beta), paste0(" - ", abs(beta)))
r2 <- round(summary(lm1)$r.squared, 5)
vform1 <- paste0("y = ", alpha, beta, "*x")
vform2 <- paste0("R^2 == ", r2)
xlab <- "Precinct-yearly criminal summons change (0.1=10%)"
p <- ggplot(dftemp, aes_string(x = X, y = Y)) +
  geom_point(alpha = 0.8, shape = 1, color = "black", size = 5) +
  geom_smooth(method = "lm", alpha = 0.2, color = "black") +
  geom_vline(xintercept = 0, linetype = "dashed", color = "grey50") +
  geom_hline(yintercept = 0, linetype = "dashed", color = "grey50") +
  labs(x = xlab, y = "Precinct-yearly crime change (0.1=10%)") +
  annotate(geom = "text", x = -0.15, y = 0.60, label = vform1, size = 8) +
  annotate(geom = "text", x = -0.15, y = 0.50, label = vform2, parse = TRUE, size = 8) +
  ylim(-0.7, 0.7) +
  plottheme  
ggsave("../Tables-Figures/FigureA7.jpg", plot = p, width = 9, height = 7,  dpi = 300)      
rm(df, dftemp, lm1, p, plottheme, alpha, beta, f1, r2, vform1, vform2, X, xlab, Y); gc()




############################################################################################-
### Plot: scatterplot summons changes between t and t-1 vs pre-intervention crime rates ####
############################################################################################-
#plot theme
plottheme <- theme(
  axis.text.x = element_text(size = 20),
  axis.text.y = element_text(size = 20),
  axis.title.x = element_text(size = 20),
  axis.title.y = element_text(size = 20),
  strip.text.x = element_text(size = 15),
  strip.background = element_rect(fill = "grey85", color = "black"),
  panel.background = element_blank(), axis.line = element_line(colour = "black"),
  legend.key = element_rect(colour = "transparent", fill = "transparent"),
  legend.key.width = unit(1.5, "cm"),
  legend.text = element_text(size = 20),
  legend.title = element_text(size = 20),
  legend.position = "none",
  rect = element_rect(fill = "transparent"))
### Summons vs pre-intervention crime rates ######-
#Aggregate at the group-precinct level (exclude precinct 22 as it is Central Park)
#group is the 12/01/year to 01/18/year+1 period
df <- fread("../Data/Crime day-year-precinct data 2006-2019.csv") %>%
  mutate(date = mdy(paste(month, day, year, sep = "-"))) %>%
  filter(precinct != 22)
for(X in 2006:2019){
  date1 <- mdy(paste0("12-01-", X)) # december 1st of each year
  date2 <- mdy(paste0("01-18-", X+1)) # january 18th of each year
  df[,paste0("i", X)] <-ifelse(df$date >= date1 & df$date <= date2, X, 0)
} 
rm(date1, date2, X)
#Precinct yearly-precinct pre-intervention crime rates (divided by 8 to get yearly rates)
dfmean <- df %>%
  filter(year <= 2013) %>%
  select(precinct, crime_major, crime_nonmajor) %>%
  group_by(precinct) %>%
  summarise_all(list(sum)) %>%
  group_by(precinct) %>%
  summarise_at(vars(crime_major, crime_nonmajor), function(x) x/8) %>%
  as.data.frame() %>%
  arrange(precinct) %>%
  rename(crime_major_pre = crime_major,
         crime_nonmajor_pre = crime_nonmajor) %>%
  left_join(df %>% distinct(precinct, population), by = "precinct") %>%
  mutate(crime_major_prepop = crime_major_pre/population*1000, 
         crime_nonmajor_prepop = crime_nonmajor_pre/population*1000)
#percentage difference by period and precinct
dftemp <- df %>%
  mutate(period = rowSums(select(., i2006:i2019))) %>%
  select(period, precinct, arrest_nonmajor, arrest_major, starts_with("crime"), summons, stopfrisk) %>%
  filter(period %in% 2006:2018) %>%
  group_by(period, precinct) %>%
  summarise_all(list(sum)) %>%
  as.data.frame() %>%
  arrange(precinct) %>%
  group_by(precinct) %>%
  mutate_at(vars(arrest_nonmajor:stopfrisk), function(x) (x - lag(x))/lag(x)) %>%
  as.data.frame() %>%
  filter(period == 2014) %>%
  mutate(period2 = paste0("dec",substr(period,3,4),"-jan",substr(period+1,3,4)," / dec", 
                          substr(period-1,3,4),"-jan", substr(period,3,4))) %>%
  left_join(dfmean, by = "precinct")
rm(dfmean)

#Plot
X <- "crime_nonmajor_prepop"
Y <- "summons"
f1 <- as.formula(paste0(Y, "~", X))
lm1 <- lm(f1, data = dftemp)
alpha <- round(unname(lm1$coefficients["(Intercept)"]), 4)
beta <- round(unname(lm1$coefficients[X]), 4)
beta <- ifelse(beta >= 0, paste0(" + ", beta), paste0(" - ", abs(beta)))
r2 <- round(summary(lm1)$r.squared, 3)
vform1 <- paste0("y = ", alpha, beta, "*x")
vform2 <- paste0("R^2 == ", r2)
ylab <- "Precinct pre-intervention criminal summons change"
xtext1 <- 80
xlim1 <- 130
p <- ggplot(dftemp %>% filter(precinct != 14), aes_string(x = X, y = Y)) +
  geom_point(alpha = 0.8, shape = 1, color = "black", size = 5) +
  geom_smooth(method = "lm", alpha = 0.2, color = "black", data = dftemp) +
  geom_vline(xintercept = 0, linetype = "dashed", color = "grey50") +
  geom_hline(yintercept = 0, linetype = "dashed", color = "grey50") +
  labs(x = "Precinct pre-intervention yearly crime rate", y = ylab) +
  annotate(geom = "text", x = xtext1, y = 0.05, label = vform1, size = 8) +
  annotate(geom = "text", x = xtext1, y = -0.05, label = vform2, parse = TRUE, size = 8) +
  coord_cartesian(xlim = c(0, xlim1), ylim = c(-0.9, 0.1)) +
  plottheme  
ggsave("../Tables-Figures/FigureA3A.jpg", plot = p, width = 9, height = 7,  dpi = 300)  

#Plot
X <- "crime_major_prepop"
Y <- "summons"
f1 <- as.formula(paste0(Y, "~", X))
lm1 <- lm(f1, data = dftemp)
alpha <- round(unname(lm1$coefficients["(Intercept)"]), 4)
beta <- round(unname(lm1$coefficients[X]), 4)
beta <- ifelse(beta >= 0, paste0(" + ", beta), paste0(" - ", abs(beta)))
r2 <- round(summary(lm1)$r.squared, 3)
vform1 <- paste0("y = ", alpha, beta, "*x")
vform2 <- paste0("R^2 == ", r2)
ylab <- "Precinct pre-intervention criminal summons change"
xtext1 <- 35
xlim1 <- 50
p <- ggplot(dftemp %>% filter(precinct != 14), aes_string(x = X, y = Y)) +
  geom_point(alpha = 0.8, shape = 1, color = "black", size = 5) +
  geom_smooth(method = "lm", alpha = 0.2, color = "black", data = dftemp) +
  geom_vline(xintercept = 0, linetype = "dashed", color = "grey50") +
  geom_hline(yintercept = 0, linetype = "dashed", color = "grey50") +
  labs(x = "Precinct pre-intervention yearly crime rate", y = ylab) +
  annotate(geom = "text", x = xtext1, y = 0.05, label = vform1, size = 8) +
  annotate(geom = "text", x = xtext1, y = -0.05, label = vform2, parse = TRUE, size = 8) +
  coord_cartesian(xlim = c(0, xlim1), ylim = c(-0.9, 0.1)) +
  plottheme  
ggsave("../Tables-Figures/FigureA3B.jpg", plot = p, width = 9, height = 7,  dpi = 300)  
rm(df, dftemp, lm1, p, plottheme, alpha, beta, f1, r2, vform1, vform2, X, xlim1, xtext1, Y, ylab); gc()




###################################################################################-
### Plot: Map changes by precinct-group level #####################################
###################################################################################-
#Precinct map
#https://hub.arcgis.com/datasets/DCP::nyc-police-precincts/data?geometry=-74.602%2C40.524%2C-73.353%2C40.888
sf <- read_sf("../Data/NYC_Police_Precincts/NYC_Police_Precincts.shp") %>%
  rename_all(tolower) 
plottheme <- theme(
  axis.text.x = element_text(size = 9),
  axis.text.y = element_text(size = 9),
  axis.title.x = element_text(size = 20),
  axis.title.y = element_text(size = 20),
  strip.text.x = element_text(size = 15),
  strip.background = element_rect(fill = "grey85", color = "black"),
  panel.background = element_blank(), axis.line = element_line(colour = "black"),
  legend.title = element_text(size = 18),
  legend.text = element_text(size = 20),
  legend.key.size = unit(1.0, "cm"),
  legend.position = c(0.15, 0.7),
  rect = element_rect(fill = "transparent"))

#Aggregate at the group-city level (exclude precinct 22 as it is Central Park)
#group is the 12/01/year to 01/18/year+1 period
df <- fread("../Data/Crime day-year-precinct data 2006-2019.csv") %>%
  mutate(date = mdy(paste(month, day, year, sep = "-"))) %>%
  filter(precinct != 22)
for(X in 2006:2019){
  date1 <- mdy(paste0("12-01-", X)) # december 1st of each year
  date2 <- mdy(paste0("01-18-", X+1)) # january 18th of next year
  df[,paste0("i", X)] <-ifelse(df$date >= date1 & df$date <= date2, X, 0)
} 
rm(date1, date2, X)
#difference by period and precinct
df <- df %>%
  mutate(period = rowSums(select(., i2006:i2019))) %>%
  select(period, precinct, arrest_nonmajor, arrest_major, starts_with("crime"), summons, stopfrisk) %>%
  #select(-crime_murder, -crime_rape) %>%
  filter(period %in% 2006:2018) %>%
  group_by(period, precinct) %>%
  summarise_all(list(sum)) %>%
  as.data.frame() %>%
  arrange(precinct) %>%
  group_by(precinct) %>%
  mutate_at(vars(arrest_nonmajor:stopfrisk), function(x) (x - lag(x))/lag(x)) %>%
  as.data.frame() %>%
  filter(period == 2014) %>%
  mutate(period2 = paste0("dec",substr(period,3,4),"-jan",substr(period+1,3,4)," / dec", 
                         substr(period-1,3,4),"-jan", substr(period,3,4)))
sftemp <- sf %>%
  left_join(df %>% filter(period == 2014), by = "precinct")

#Plot
X <- "summons"
p <- ggplot(sftemp) +
  geom_sf(aes_string(fill = X), size = 0.01) +
  labs(fill = "Change\n(0.1=10%)") +
  scale_fill_gradient2(low = "darkblue", high = "darkgreen") +
  plottheme
ggsave("../Tables-Figures/Figure3A.jpg", plot = p, width = 7, height = 7, dpi = 300)  

#Plot
X <- "arrest_nonmajor"
p <- ggplot(sftemp) +
  geom_sf(aes_string(fill = X), size = 0.01) +
  labs(fill = "Change\n(0.1=10%)") +
  scale_fill_gradient2(low = "darkblue", high = "darkgreen") +
  plottheme
ggsave("../Tables-Figures/FigureA2A.jpg", plot = p, width = 7, height = 7, dpi = 300)  

#Plot
X <- "crime_nonmajor"
p <- ggplot(sftemp) +
  geom_sf(aes_string(fill = X), size = 0.01) +
  labs(fill = "Change\n(0.1=10%)") +
  scale_fill_gradient2(low = "darkblue", high = "darkgreen") +
  plottheme
ggsave("../Tables-Figures/FigureA2B.jpg", plot = p, width = 7, height = 7, dpi = 300)  

#Plot
X <- "crime_nonmajor_in"
p <- ggplot(sftemp) +
  geom_sf(aes_string(fill = X), size = 0.01) +
  labs(fill = "Change\n(0.1=10%)") +
  scale_fill_gradient2(low = "darkblue", high = "darkgreen") +
  plottheme
ggsave("../Tables-Figures/FigureA2C.jpg", plot = p, width = 7, height = 7, dpi = 300)  

#Plot
X <- "crime_nonmajor_out"
p <- ggplot(sftemp) +
  geom_sf(aes_string(fill = X), size = 0.01) +
  labs(fill = "Change\n(0.1=10%)") +
  scale_fill_gradient2(low = "darkblue", high = "darkgreen") +
  plottheme
ggsave("../Tables-Figures/FigureA2D.jpg", plot = p, width = 7, height = 7, dpi = 300)  
rm(df, p, plottheme, sf, sftemp, X); gc()





######################################################################################-
### Plot: coefficients Negative binomial between t and t-1 (authors replications) ####
######################################################################################-
#Aggregate at the group-city level (exclude precinct 22 as it is Central Park)
df <- fread("../Data/Negative binomial day-period estimates.csv") %>%
  mutate(group = ifelse((ci.low-1)*(ci.up-1) > 0, "significant", "not significant"), 
         group = ifelse(period.treat == "dec14-jan15", "main", group)) %>%
  filter(fe == "no.fe")
plottheme <- theme(
  axis.text.x = element_text(size = 18),
  axis.text.y = element_text(size = 18),
  axis.title.x = element_text(size = 20),
  axis.title.y = element_text(size = 20),
  strip.text.x = element_text(size = 15),
  strip.background = element_rect(fill = "grey85", color = "black"),
  panel.background = element_blank(), axis.line = element_line(colour = "black"),
  legend.key = element_rect(colour = "transparent", fill = "transparent"),
  legend.key.width = unit(0.5, "cm"),
  legend.text = element_text(size = 20),
  legend.title = element_text(size = 20),
  legend.position = "top",
  rect = element_rect(fill = "transparent"))

#Plot
X <- "summons"
xlim <- 1.35
xsep <- 0.4
xtext <- 1.1
xup <- 1.3+xlim
xend <- ifelse(1-xlim < 0, 0, 1-xlim)
dftemp <- df %>% 
  filter(outcome == X) %>% 
  group_by(period.treat) %>% 
  mutate(id = cur_group_id(),
         text = paste0(format(round(irr, 3), nsmall = 3), " (", format(round(se, 3), nsmall = 3), ")"))
p <- ggplot(dftemp, aes(x = irr, y = id)) +
  geom_vline(xintercept = 1, linetype = "dashed", color = "grey10") +
  geom_pointrange(aes(xmin = ci.low, xmax = ci.up), size = 0.5) +
  geom_point(aes(fill = group, shape = group), size = 5) +
  labs(y = "Treatment window period", x = "Incident Rate Ratio", fill = NULL, shape = NULL) +
  scale_x_continuous(breaks = round(seq(xend, 1+xlim, xsep), 1), limits = c(xend, xup)) +
  scale_y_continuous(breaks = 1:12, labels = dftemp$period.treat, limits = c(1, 12.5)) +
  annotate(geom = "text", x = xtext+xlim, y = 1:12, label = paste0(dftemp$text), fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 12.5, label = "IRR   (SE)", fontface = "bold", size = 6) +
  scale_shape_manual(values = c(21,23,24)) +
  scale_fill_manual(values = c("darkred", "grey", "green4")) +
  plottheme
ggsave("../Tables-Figures/Figure2A.jpg", plot = p, width = 8, height = 7,  dpi = 300)  

#Plot
X <- "crime_major"
xlim <- 0.3
xsep <- 0.2
xtext <- 1.12
xup <- 1.2+xlim
xend <- ifelse(1-xlim < 0, 0, 1-xlim)
dftemp <- df %>% 
  filter(outcome == X) %>% 
  group_by(period.treat) %>% 
  mutate(id = cur_group_id(),
         text = paste0(format(round(irr, 3), nsmall = 3), " (", format(round(se, 3), nsmall = 3), ")"))
p <- ggplot(dftemp, aes(x = irr, y = id)) +
  geom_vline(xintercept = 1, linetype = "dashed", color = "grey10") +
  geom_pointrange(aes(xmin = ci.low, xmax = ci.up), size = 0.5) +
  geom_point(aes(fill = group, shape = group), size = 5) +
  labs(y = "Treatment window period", x = "Incident Rate Ratio", fill = NULL, shape = NULL) +
  scale_x_continuous(breaks = round(seq(xend, 1+xlim, xsep), 1), limits = c(xend, xup)) +
  scale_y_continuous(breaks = 1:12, labels = dftemp$period.treat, limits = c(1, 12.5)) +
  annotate(geom = "text", x = xtext+xlim, y = 1:12, label = paste0(dftemp$text), fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 12.5, label = "IRR   (SE)", fontface = "bold", size = 6) +
  scale_shape_manual(values = c(21,23,24)) +
  scale_fill_manual(values = c("darkred", "grey", "green4")) +
  plottheme
ggsave("../Tables-Figures/Figure2B.jpg", plot = p, width = 8, height = 7,  dpi = 300)  

#Plot
X <- "crime_major_in"
xlim <- 0.3
xsep <- 0.2
xtext <- 1.12
xup <- 1.2+xlim
xend <- ifelse(1-xlim < 0, 0, 1-xlim)
dftemp <- df %>% 
  filter(outcome == X) %>% 
  group_by(period.treat) %>% 
  mutate(id = cur_group_id(),
         text = paste0(format(round(irr, 3), nsmall = 3), " (", format(round(se, 3), nsmall = 3), ")"))
p <- ggplot(dftemp, aes(x = irr, y = id)) +
  geom_vline(xintercept = 1, linetype = "dashed", color = "grey10") +
  geom_pointrange(aes(xmin = ci.low, xmax = ci.up), size = 0.5) +
  geom_point(aes(fill = group, shape = group), size = 5) +
  labs(y = "Treatment window period", x = "Incident Rate Ratio", fill = NULL, shape = NULL) +
  scale_x_continuous(breaks = round(seq(xend, 1+xlim, xsep), 1), limits = c(xend, xup)) +
  scale_y_continuous(breaks = 1:12, labels = dftemp$period.treat, limits = c(1, 12.5)) +
  annotate(geom = "text", x = xtext+xlim, y = 1:12, label = paste0(dftemp$text), fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 12.5, label = "IRR   (SE)", fontface = "bold", size = 6) +
  scale_shape_manual(values = c(21,23,24)) +
  scale_fill_manual(values = c("darkred", "grey", "green4")) +
  plottheme
ggsave("../Tables-Figures/Figure4A.jpg", plot = p, width = 8, height = 7,  dpi = 300)  

#Plot
X <- "crime_major_out"
xlim <- 0.3
xsep <- 0.2
xtext <- 1.12
xup <- 1.2+xlim
xend <- ifelse(1-xlim < 0, 0, 1-xlim)
dftemp <- df %>% 
  filter(outcome == X) %>% 
  group_by(period.treat) %>% 
  mutate(id = cur_group_id(),
         text = paste0(format(round(irr, 3), nsmall = 3), " (", format(round(se, 3), nsmall = 3), ")"))
p <- ggplot(dftemp, aes(x = irr, y = id)) +
  geom_vline(xintercept = 1, linetype = "dashed", color = "grey10") +
  geom_pointrange(aes(xmin = ci.low, xmax = ci.up), size = 0.5) +
  geom_point(aes(fill = group, shape = group), size = 5) +
  labs(y = "Treatment window period", x = "Incident Rate Ratio", fill = NULL, shape = NULL) +
  scale_x_continuous(breaks = round(seq(xend, 1+xlim, xsep), 1), limits = c(xend, xup)) +
  scale_y_continuous(breaks = 1:12, labels = dftemp$period.treat, limits = c(1, 12.5)) +
  annotate(geom = "text", x = xtext+xlim, y = 1:12, label = paste0(dftemp$text), fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 12.5, label = "IRR   (SE)", fontface = "bold", size = 6) +
  scale_shape_manual(values = c(21,23,24)) +
  scale_fill_manual(values = c("darkred", "grey", "green4")) +
  plottheme
ggsave("../Tables-Figures/Figure4B.jpg", plot = p, width = 8, height = 7,  dpi = 300)  

#Plot
X <- "arrest_nonmajor"
xlim <- 0.6
xsep <- 0.2
xtext <- 1.18
xup <- 1.3+xlim
xend <- ifelse(1-xlim < 0, 0, 1-xlim)
dftemp <- df %>% 
  filter(outcome == X) %>% 
  group_by(period.treat) %>% 
  mutate(id = cur_group_id(),
         text = paste0(format(round(irr, 3), nsmall = 3), " (", format(round(se, 3), nsmall = 3), ")"))
p <- ggplot(dftemp, aes(x = irr, y = id)) +
  geom_vline(xintercept = 1, linetype = "dashed", color = "grey10") +
  geom_pointrange(aes(xmin = ci.low, xmax = ci.up), size = 0.5) +
  geom_point(aes(fill = group, shape = group), size = 5) +
  labs(y = "Treatment window period", x = "Incident Rate Ratio", fill = NULL, shape = NULL) +
  scale_x_continuous(breaks = round(seq(xend, 1+xlim, xsep), 1), limits = c(xend, xup)) +
  scale_y_continuous(breaks = 1:12, labels = dftemp$period.treat, limits = c(1, 12.5)) +
  annotate(geom = "text", x = xtext+xlim, y = 1:12, label = paste0(dftemp$text), fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 12.5, label = "IRR   (SE)", fontface = "bold", size = 6) +
  scale_shape_manual(values = c(21,23,24)) +
  scale_fill_manual(values = c("darkred", "grey", "green4")) +
  plottheme
ggsave("../Tables-Figures/FigureA1A.jpg", plot = p, width = 8, height = 7,  dpi = 300)  

#Plot
X <- "crime_nonmajor"
xlim <- 0.3
xsep <- 0.2
xtext <- 1.12
xup <- 1.2+xlim
xend <- ifelse(1-xlim < 0, 0, 1-xlim)
dftemp <- df %>% 
  filter(outcome == X) %>% 
  group_by(period.treat) %>% 
  mutate(id = cur_group_id(),
         text = paste0(format(round(irr, 3), nsmall = 3), " (", format(round(se, 3), nsmall = 3), ")"))
p <- ggplot(dftemp, aes(x = irr, y = id)) +
  geom_vline(xintercept = 1, linetype = "dashed", color = "grey10") +
  geom_pointrange(aes(xmin = ci.low, xmax = ci.up), size = 0.5) +
  geom_point(aes(fill = group, shape = group), size = 5) +
  labs(y = "Treatment window period", x = "Incident Rate Ratio", fill = NULL, shape = NULL) +
  scale_x_continuous(breaks = round(seq(xend, 1+xlim, xsep), 1), limits = c(xend, xup)) +
  scale_y_continuous(breaks = 1:12, labels = dftemp$period.treat, limits = c(1, 12.5)) +
  annotate(geom = "text", x = xtext+xlim, y = 1:12, label = paste0(dftemp$text), fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 12.5, label = "IRR   (SE)", fontface = "bold", size = 6) +
  scale_shape_manual(values = c(21,23,24)) +
  scale_fill_manual(values = c("darkred", "grey", "green4")) +
  plottheme
ggsave("../Tables-Figures/FigureA1B.jpg", plot = p, width = 8, height = 7,  dpi = 300)  

#Plot
X <- "crime_nonmajor_in"
xlim <- 0.3
xsep <- 0.2
xtext <- 1.12
xup <- 1.2+xlim
xend <- ifelse(1-xlim < 0, 0, 1-xlim)
dftemp <- df %>% 
  filter(outcome == X) %>% 
  group_by(period.treat) %>% 
  mutate(id = cur_group_id(),
         text = paste0(format(round(irr, 3), nsmall = 3), " (", format(round(se, 3), nsmall = 3), ")"))
p <- ggplot(dftemp, aes(x = irr, y = id)) +
  geom_vline(xintercept = 1, linetype = "dashed", color = "grey10") +
  geom_pointrange(aes(xmin = ci.low, xmax = ci.up), size = 0.5) +
  geom_point(aes(fill = group, shape = group), size = 5) +
  labs(y = "Treatment window period", x = "Incident Rate Ratio", fill = NULL, shape = NULL) +
  scale_x_continuous(breaks = round(seq(xend, 1+xlim, xsep), 1), limits = c(xend, xup)) +
  scale_y_continuous(breaks = 1:12, labels = dftemp$period.treat, limits = c(1, 12.5)) +
  annotate(geom = "text", x = xtext+xlim, y = 1:12, label = paste0(dftemp$text), fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 12.5, label = "IRR   (SE)", fontface = "bold", size = 6) +
  scale_shape_manual(values = c(21,23,24)) +
  scale_fill_manual(values = c("darkred", "grey", "green4")) +
  plottheme
ggsave("../Tables-Figures/FigureA1C.jpg", plot = p, width = 8, height = 7,  dpi = 300)  

#Plot
X <- "crime_nonmajor_out"
xlim <- 0.3
xsep <- 0.2
xtext <- 1.12
xup <- 1.2+xlim
xend <- ifelse(1-xlim < 0, 0, 1-xlim)
dftemp <- df %>% 
  filter(outcome == X) %>% 
  group_by(period.treat) %>% 
  mutate(id = cur_group_id(),
         text = paste0(format(round(irr, 3), nsmall = 3), " (", format(round(se, 3), nsmall = 3), ")"))
p <- ggplot(dftemp, aes(x = irr, y = id)) +
  geom_vline(xintercept = 1, linetype = "dashed", color = "grey10") +
  geom_pointrange(aes(xmin = ci.low, xmax = ci.up), size = 0.5) +
  geom_point(aes(fill = group, shape = group), size = 5) +
  labs(y = "Treatment window period", x = "Incident Rate Ratio", fill = NULL, shape = NULL) +
  scale_x_continuous(breaks = round(seq(xend, 1+xlim, xsep), 1), limits = c(xend, xup)) +
  scale_y_continuous(breaks = 1:12, labels = dftemp$period.treat, limits = c(1, 12.5)) +
  annotate(geom = "text", x = xtext+xlim, y = 1:12, label = paste0(dftemp$text), fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 12.5, label = "IRR   (SE)", fontface = "bold", size = 6) +
  scale_shape_manual(values = c(21,23,24)) +
  scale_fill_manual(values = c("darkred", "grey", "green4")) +
  plottheme
ggsave("../Tables-Figures/FigureA1D.jpg", plot = p, width = 8, height = 7,  dpi = 300)  
rm(df, dftemp, p, plottheme, X, xend, xlim, xsep, xtext, xup); gc()




#######################################################################################################-
### Plot: coefficients Negative binomial between t and t-1 (using Dec/22 as starting treat window) ####
#######################################################################################################-
#Aggregate at the group-city level (exclude precint 22 as it is Central Park)
df <- fread("../Data/Negative binomial day-period estimates robustness.csv") %>%
  mutate(group = ifelse((ci.low-1)*(ci.up-1) > 0, "significant", "not significant"), 
         group = ifelse(period.treat == "dec14-jan15", "main", group)) %>%
  filter(fe == "no.fe")
plottheme <- theme(
  axis.text.x = element_text(size = 18),
  axis.text.y = element_text(size = 18),
  axis.title.x = element_text(size = 20),
  axis.title.y = element_text(size = 20),
  strip.text.x = element_text(size = 15),
  strip.background = element_rect(fill = "grey85", color = "black"),
  panel.background = element_blank(), axis.line = element_line(colour = "black"),
  legend.key = element_rect(colour = "transparent", fill = "transparent"),
  legend.key.width = unit(0.5, "cm"),
  legend.text = element_text(size = 20),
  legend.title = element_text(size = 20),
  legend.position = "top",
  rect = element_rect(fill = "transparent"))

#Plot
X <- "summons"
xlim <- 2.0
xsep <- 0.4
xtext <- 1.28
xup <- 1.6+xlim
xend <- ifelse(1-xlim < 0, 0, 1-xlim)
dftemp <- df %>% 
  filter(outcome == X) %>% 
  group_by(period.treat) %>% 
  mutate(id = cur_group_id(),
         text = paste0(format(round(irr, 3), nsmall = 3), " (", format(round(se, 3), nsmall = 3), ")"))
p <- ggplot(dftemp, aes(x = irr, y = id)) +
  geom_vline(xintercept = 1, linetype = "dashed", color = "grey10") +
  geom_pointrange(aes(xmin = ci.low, xmax = ci.up), size = 0.5) +
  geom_point(aes(fill = group, shape = group), size = 5) +
  labs(y = "Treatment window period", x = "Incident Rate Ratio", fill = NULL, shape = NULL) +
  scale_x_continuous(breaks = round(seq(xend, 1+xlim, xsep), 1), limits = c(xend, xup)) +
  scale_y_continuous(breaks = 1:12, labels = dftemp$period.treat, limits = c(1, 12.5)) +
  annotate(geom = "text", x = xtext+xlim, y = 1:12, label = paste0(dftemp$text), fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 12.5, label = "IRR   (SE)", fontface = "bold", size = 6) +
  scale_shape_manual(values = c(21,23,24)) +
  scale_fill_manual(values = c("darkred", "grey", "green4")) +
  plottheme
ggsave("../Tables-Figures/Figure5A.jpg", plot = p, width = 8, height = 7,  dpi = 300)  

#Plot
X <- "crime_major"
xlim <- 0.3
xsep <- 0.2
xtext <- 1.12
xup <- 1.2+xlim
xend <- ifelse(1-xlim < 0, 0, 1-xlim)
dftemp <- df %>% 
  filter(outcome == X) %>% 
  group_by(period.treat) %>% 
  mutate(id = cur_group_id(),
         text = paste0(format(round(irr, 3), nsmall = 3), " (", format(round(se, 3), nsmall = 3), ")"))
p <- ggplot(dftemp, aes(x = irr, y = id)) +
  geom_vline(xintercept = 1, linetype = "dashed", color = "grey10") +
  geom_pointrange(aes(xmin = ci.low, xmax = ci.up), size = 0.5) +
  geom_point(aes(fill = group, shape = group), size = 5) +
  labs(y = "Treatment window period", x = "Incident Rate Ratio", fill = NULL, shape = NULL) +
  scale_x_continuous(breaks = round(seq(xend, 1+xlim, xsep), 1), limits = c(xend, xup)) +
  scale_y_continuous(breaks = 1:12, labels = dftemp$period.treat, limits = c(1, 12.5)) +
  annotate(geom = "text", x = xtext+xlim, y = 1:12, label = paste0(dftemp$text), fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 12.5, label = "IRR   (SE)", fontface = "bold", size = 6) +
  scale_shape_manual(values = c(21,23,24)) +
  scale_fill_manual(values = c("darkred", "grey", "green4")) +
  plottheme
ggsave("../Tables-Figures/Figure5B.jpg", plot = p, width = 8, height = 7,  dpi = 300)  

#Plot
X <- "crime_major_in"
xlim <- 0.3
xsep <- 0.2
xtext <- 1.12
xup <- 1.2+xlim
xend <- ifelse(1-xlim < 0, 0, 1-xlim)
dftemp <- df %>% 
  filter(outcome == X) %>% 
  group_by(period.treat) %>% 
  mutate(id = cur_group_id(),
         text = paste0(format(round(irr, 3), nsmall = 3), " (", format(round(se, 3), nsmall = 3), ")"))
p <- ggplot(dftemp, aes(x = irr, y = id)) +
  geom_vline(xintercept = 1, linetype = "dashed", color = "grey10") +
  geom_pointrange(aes(xmin = ci.low, xmax = ci.up), size = 0.5) +
  geom_point(aes(fill = group, shape = group), size = 5) +
  labs(y = "Treatment window period", x = "Incident Rate Ratio", fill = NULL, shape = NULL) +
  scale_x_continuous(breaks = round(seq(xend, 1+xlim, xsep), 1), limits = c(xend, xup)) +
  scale_y_continuous(breaks = 1:12, labels = dftemp$period.treat, limits = c(1, 12.5)) +
  annotate(geom = "text", x = xtext+xlim, y = 1:12, label = paste0(dftemp$text), fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 12.5, label = "IRR   (SE)", fontface = "bold", size = 6) +
  scale_shape_manual(values = c(21,23,24)) +
  scale_fill_manual(values = c("darkred", "grey", "green4")) +
  plottheme
ggsave("../Tables-Figures/Figure5C.jpg", plot = p, width = 8, height = 7,  dpi = 300)  

#Plot
X <- "crime_major_out"
xlim <- 0.3
xsep <- 0.2
xtext <- 1.12
xup <- 1.2+xlim
xend <- ifelse(1-xlim < 0, 0, 1-xlim)
dftemp <- df %>% 
  filter(outcome == X) %>% 
  group_by(period.treat) %>% 
  mutate(id = cur_group_id(),
         text = paste0(format(round(irr, 3), nsmall = 3), " (", format(round(se, 3), nsmall = 3), ")"))
p <- ggplot(dftemp, aes(x = irr, y = id)) +
  geom_vline(xintercept = 1, linetype = "dashed", color = "grey10") +
  geom_pointrange(aes(xmin = ci.low, xmax = ci.up), size = 0.5) +
  geom_point(aes(fill = group, shape = group), size = 5) +
  labs(y = "Treatment window period", x = "Incident Rate Ratio", fill = NULL, shape = NULL) +
  scale_x_continuous(breaks = round(seq(xend, 1+xlim, xsep), 1), limits = c(xend, xup)) +
  scale_y_continuous(breaks = 1:12, labels = dftemp$period.treat, limits = c(1, 12.5)) +
  annotate(geom = "text", x = xtext+xlim, y = 1:12, label = paste0(dftemp$text), fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 12.5, label = "IRR   (SE)", fontface = "bold", size = 6) +
  scale_shape_manual(values = c(21,23,24)) +
  scale_fill_manual(values = c("darkred", "grey", "green4")) +
  plottheme
ggsave("../Tables-Figures/Figure5D.jpg", plot = p, width = 8, height = 7,  dpi = 300)  
rm(df, dftemp, p, plottheme, X, xend, xlim, xsep, xtext, xup); gc()







######################################################################################-
### Plot: coefficients Negative binomial between t and t-1 (census tract level) ######
######################################################################################-
#Aggregate at the group-city level (exclude precint 22 as it is Central Park)
df <- fread("../Data/Negative binomial day-period estimates tract level.csv") %>%
  mutate(group = ifelse((ci.low-1)*(ci.up-1) > 0, "significant", "not significant"), 
         group = ifelse(period.treat == "dec14-jan15", "main", group)) %>%
  filter(fe == "no.fe")
plottheme <- theme(
  axis.text.x = element_text(size = 18),
  axis.text.y = element_text(size = 18),
  axis.title.x = element_text(size = 20),
  axis.title.y = element_text(size = 20),
  strip.text.x = element_text(size = 15),
  strip.background = element_rect(fill = "grey85", color = "black"),
  panel.background = element_blank(), axis.line = element_line(colour = "black"),
  legend.key = element_rect(colour = "transparent", fill = "transparent"),
  legend.key.width = unit(0.5, "cm"),
  legend.text = element_text(size = 20),
  legend.title = element_text(size = 20),
  legend.position = "top",
  rect = element_rect(fill = "transparent"))

#Plot
X <- "summons"
xlim <- 1.35
xsep <- 0.4
xtext <- 1.1
xup <- 1.3+xlim
xend <- ifelse(1-xlim < 0, 0, 1-xlim)
dftemp <- df %>% 
  filter(outcome == X) %>% 
  group_by(period.treat) %>% 
  mutate(id = cur_group_id(),
         text = paste0(format(round(irr, 3), nsmall = 3), " (", format(round(se, 3), nsmall = 3), ")"))
p <- ggplot(dftemp, aes(x = irr, y = id)) +
  geom_vline(xintercept = 1, linetype = "dashed", color = "grey10") +
  geom_pointrange(aes(xmin = ci.low, xmax = ci.up), size = 0.5) +
  geom_point(aes(fill = group, shape = group), size = 5) +
  labs(y = "Treatment window period", x = "Incident Rate Ratio", fill = NULL, shape = NULL) +
  scale_x_continuous(breaks = round(seq(xend, 1+xlim, xsep), 1), limits = c(xend, xup)) +
  scale_y_continuous(breaks = 1:12, labels = dftemp$period.treat, limits = c(1, 12.5)) +
  annotate(geom = "text", x = xtext+xlim, y = 1:12, label = paste0(dftemp$text), fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 12.5, label = "IRR   (SE)", fontface = "bold", size = 6) +
  scale_shape_manual(values = c(21,23,24)) +
  scale_fill_manual(values = c("darkred", "grey", "green4")) +
  plottheme
ggsave("../Tables-Figures/FigureA8A.jpg", plot = p, width = 8, height = 7,  dpi = 300)  

#Plot
X <- "crime_major"
xlim <- 0.3
xsep <- 0.2
xtext <- 1.12
xup <- 1.2+xlim
xend <- ifelse(1-xlim < 0, 0, 1-xlim)
dftemp <- df %>% 
  filter(outcome == X) %>% 
  group_by(period.treat) %>% 
  mutate(id = cur_group_id(),
         text = paste0(format(round(irr, 3), nsmall = 3), " (", format(round(se, 3), nsmall = 3), ")"))
p <- ggplot(dftemp, aes(x = irr, y = id)) +
  geom_vline(xintercept = 1, linetype = "dashed", color = "grey10") +
  geom_pointrange(aes(xmin = ci.low, xmax = ci.up), size = 0.5) +
  geom_point(aes(fill = group, shape = group), size = 5) +
  labs(y = "Treatment window period", x = "Incident Rate Ratio", fill = NULL, shape = NULL) +
  scale_x_continuous(breaks = round(seq(xend, 1+xlim, xsep), 1), limits = c(xend, xup)) +
  scale_y_continuous(breaks = 1:12, labels = dftemp$period.treat, limits = c(1, 12.5)) +
  annotate(geom = "text", x = xtext+xlim, y = 1:12, label = paste0(dftemp$text), fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 12.5, label = "IRR   (SE)", fontface = "bold", size = 6) +
  scale_shape_manual(values = c(21,23,24)) +
  scale_fill_manual(values = c("darkred", "grey", "green4")) +
  plottheme
ggsave("../Tables-Figures/FigureA8B.jpg", plot = p, width = 8, height = 7,  dpi = 300)  

#Plot
X <- "crime_major_in"
xlim <- 0.3
xsep <- 0.2
xtext <- 1.12
xup <- 1.2+xlim
xend <- ifelse(1-xlim < 0, 0, 1-xlim)
dftemp <- df %>% 
  filter(outcome == X) %>% 
  group_by(period.treat) %>% 
  mutate(id = cur_group_id(),
         text = paste0(format(round(irr, 3), nsmall = 3), " (", format(round(se, 3), nsmall = 3), ")"))
p <- ggplot(dftemp, aes(x = irr, y = id)) +
  geom_vline(xintercept = 1, linetype = "dashed", color = "grey10") +
  geom_pointrange(aes(xmin = ci.low, xmax = ci.up), size = 0.5) +
  geom_point(aes(fill = group, shape = group), size = 5) +
  labs(y = "Treatment window period", x = "Incident Rate Ratio", fill = NULL, shape = NULL) +
  scale_x_continuous(breaks = round(seq(xend, 1+xlim, xsep), 1), limits = c(xend, xup)) +
  scale_y_continuous(breaks = 1:12, labels = dftemp$period.treat, limits = c(1, 12.5)) +
  annotate(geom = "text", x = xtext+xlim, y = 1:12, label = paste0(dftemp$text), fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 12.5, label = "IRR   (SE)", fontface = "bold", size = 6) +
  scale_shape_manual(values = c(21,23,24)) +
  scale_fill_manual(values = c("darkred", "grey", "green4")) +
  plottheme
ggsave("../Tables-Figures/FigureA8C.jpg", plot = p, width = 8, height = 7,  dpi = 300)  

#Plot
X <- "crime_major_out"
xlim <- 0.3
xsep <- 0.2
xtext <- 1.12
xup <- 1.2+xlim
xend <- ifelse(1-xlim < 0, 0, 1-xlim)
dftemp <- df %>% 
  filter(outcome == X) %>% 
  group_by(period.treat) %>% 
  mutate(id = cur_group_id(),
         text = paste0(format(round(irr, 3), nsmall = 3), " (", format(round(se, 3), nsmall = 3), ")"))
p <- ggplot(dftemp, aes(x = irr, y = id)) +
  geom_vline(xintercept = 1, linetype = "dashed", color = "grey10") +
  geom_pointrange(aes(xmin = ci.low, xmax = ci.up), size = 0.5) +
  geom_point(aes(fill = group, shape = group), size = 5) +
  labs(y = "Treatment window period", x = "Incident Rate Ratio", fill = NULL, shape = NULL) +
  scale_x_continuous(breaks = round(seq(xend, 1+xlim, xsep), 1), limits = c(xend, xup)) +
  scale_y_continuous(breaks = 1:12, labels = dftemp$period.treat, limits = c(1, 12.5)) +
  annotate(geom = "text", x = xtext+xlim, y = 1:12, label = paste0(dftemp$text), fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 12.5, label = "IRR   (SE)", fontface = "bold", size = 6) +
  scale_shape_manual(values = c(21,23,24)) +
  scale_fill_manual(values = c("darkred", "grey", "green4")) +
  plottheme
ggsave("../Tables-Figures/FigureA8D.jpg", plot = p, width = 8, height = 7,  dpi = 300)  
rm(df, dftemp, p, plottheme, X, xend, xlim, xsep, xtext, xup); gc()





######################################################################################-
### Plot: coefficients Negative binomial between t and t-1 (two-way clustering) ######
######################################################################################-
#Aggregate at the group-city level (exclude precint 22 as it is Central Park)
df <- fread("../Data/Negative binomial day-period estimates.csv") %>%
  filter(fe == "no.fe") %>%
  bind_rows(fread("../Data/Negative binomial day-period estimates twoway.csv")) %>%
  mutate(group = ifelse((ci.low-1)*(ci.up-1) > 0, "significant", "not significant"), 
         group = ifelse(period.treat == "dec14-jan15", "main", group)) %>%
  mutate(sample = ifelse(fe %in% "no.fe", "one-way", "two-way"))
plottheme <- theme(
  axis.text.x = element_text(size = 18),
  axis.text.y = element_text(size = 18),
  axis.title.x = element_text(size = 20),
  axis.title.y = element_text(size = 20),
  strip.text.x = element_text(size = 15),
  strip.background = element_rect(fill = "grey85", color = "black"),
  panel.background = element_blank(), axis.line = element_line(colour = "black"),
  legend.key = element_rect(colour = "transparent", fill = "transparent"),
  legend.key.width = unit(0.5, "cm"),
  legend.text = element_text(size = 20),
  legend.title = element_text(size = 20),
  legend.position = "top",
  rect = element_rect(fill = "transparent"))

#Plot
X <- "summons"
xlim <- 1.5
xsep <- 0.4
xtext <- 2.0
xup <- 2.9+xlim
xend <- ifelse(1-xlim < 0, 0, 1-xlim)
dftemp <- df %>% 
  filter(outcome == X) %>% 
  group_by(period.treat) %>% 
  mutate(id = cur_group_id(),
         id = ifelse(sample == "one-way", id - 0.1, id + 0.1),
         setext = paste0(" (", format(round(se, 3), nsmall = 3), ")", 
                         " (", format(round(lead(se), 3), nsmall = 3), ")"), 
         text = paste0(format(round(irr, 3), nsmall = 3), setext))
p <- ggplot(dftemp, aes(x = irr, y = id, group = sample)) +
  geom_vline(xintercept = 1, linetype = "dashed", color = "grey10") +
  geom_pointrange(aes(xmin = ci.low, xmax = ci.up), size = 0.5) +
  geom_point(aes(fill = group, shape = group), size = 4) +
  labs(y = "Treatment window period", x = "Incident Rate Ratio", fill = NULL, shape = NULL) +
  scale_x_continuous(breaks = round(seq(xend, 1+xlim, xsep), 1), limits = c(xend, xup)) +
  scale_y_continuous(breaks = 1:12, labels = unique(dftemp$period.treat), limits = c(0.8, 13.5)) +
  annotate(geom = "text", x = xtext+xlim, y = 1:12, label = paste0(dftemp[dftemp$sample=="one-way",]$text), fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 13.5, label = "SE1: precinct        ", fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 13.0, label = "SE2: precinct & year", fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 12.5, label = "IRR   (SE1)   (SE2)", fontface = "bold", size = 6) +
  scale_shape_manual(values = c(21,23,24)) +
  scale_fill_manual(values = c("darkred", "grey", "green4")) +
  plottheme
ggsave("../Tables-Figures/FigureA10A.jpg", plot = p, width = 8, height = 11, dpi = 300)  

#Plot
X <- "crime_major"
xlim <- 0.3
xsep <- 0.2
xtext <- 1.4
xup <- 1.6+xlim
xend <- ifelse(1-xlim < 0, 0, 1-xlim)
dftemp <- df %>% 
  filter(outcome == X) %>% 
  group_by(period.treat) %>% 
  mutate(id = cur_group_id(),
         id = ifelse(sample == "one-way", id - 0.1, id + 0.1),
         setext = paste0(" (", format(round(se, 3), nsmall = 3), ")", 
                         " (", format(round(lead(se), 3), nsmall = 3), ")"), 
         text = paste0(format(round(irr, 3), nsmall = 3), setext))
p <- ggplot(dftemp, aes(x = irr, y = id, group = sample)) +
  geom_vline(xintercept = 1, linetype = "dashed", color = "grey10") +
  geom_pointrange(aes(xmin = ci.low, xmax = ci.up), size = 0.5) +
  geom_point(aes(fill = group, shape = group), size = 4) +
  labs(y = "Treatment window period", x = "Incident Rate Ratio", fill = NULL, shape = NULL) +
  scale_x_continuous(breaks = round(seq(xend, 1+xlim, xsep), 1), limits = c(xend, xup)) +
  scale_y_continuous(breaks = 1:12, labels = unique(dftemp$period.treat), limits = c(0.8, 13.5)) +
  annotate(geom = "text", x = xtext+xlim, y = 1:12, label = paste0(dftemp[dftemp$sample=="one-way",]$text), fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 13.5, label = "SE1: precinct        ", fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 13.0, label = "SE2: precinct & year", fontface = "bold", size = 6) +
  annotate(geom = "text", x = xtext+xlim, y = 12.5, label = "IRR   (SE1)   (SE2)", fontface = "bold", size = 6) +
  scale_shape_manual(values = c(21,23,24)) +
  scale_fill_manual(values = c("darkred", "grey", "green4")) +
  plottheme
ggsave("../Tables-Figures/FigureA10B.jpg", plot = p, width = 8, height = 11, dpi = 300)  
rm(df, dftemp, p, plottheme, X, xend, xlim, xsep, xtext, xup); gc()




#######################################################################################################-
### Plot: event study estimates (main specification and high-low exposure using 2010-2015 period) #####
#######################################################################################################-
#plot theme
plottheme <- theme(
  axis.text.x = element_text(size = 18),
  axis.text.y = element_text(size = 18),
  axis.title.x = element_text(size = 20),
  axis.title.y = element_text(size = 20),
  strip.text.x = element_text(size = 15),
  strip.background = element_rect(fill = "grey85", color = "black"),
  panel.background = element_blank(), axis.line = element_line(colour = "black"),
  legend.key = element_rect(colour = "transparent", fill = "transparent"),
  legend.key.width = unit(0.5, "cm"),
  legend.text = element_text(size = 20),
  legend.title = element_text(size = 20),
  legend.position = "top",
  rect = element_rect(fill = "transparent"))

#High vs low exposure
df <- fread("../Data/Negative binomial week-period estimates event study highlow longperiod.csv") 
df <- df %>%
  bind_rows(data.frame("outcome" = "crime_major", 
                       "week" = -1,
                       "irr" = 1, 
                       "se" = 0,
                       "ci.low" = 0, 
                       "ci.up" = 0))
p <- ggplot(df, aes(x = week, y = irr)) +
  geom_hline(yintercept = 1, linetype = "dashed", color = "grey50") +
  geom_vline(xintercept = -1, linetype = "dashed", color = "grey50") +
  geom_pointrange(aes(ymin = ci.low, ymax = ci.up), color = "darkblue") +
  geom_point(color = "darkblue", size = 3)  +
  labs(x = "Week relative to the start of the treatment window", y = "IRR (95% CI)", color = NULL, shape = NULL) +
  coord_cartesian(ylim = c(0.6, 1.4)) +
  plottheme  
ggsave("../Tables-Figures/FigureA6.jpg", plot = p, width = 8, height = 7,  dpi = 300)  
rm(df, p, plottheme); gc()





